#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.forms.models import model_to_dict
from django.http import QueryDict
from django.utils.functional import cached_property
from django.utils.http import urlencode

from paypal.utils import warn_untested

try:
    from idmapper.models import SharedMemoryModel as Model
except ImportError:
    Model = models.Model


class PayPalNVP(Model):
    """Record of a NVP interaction with PayPal."""
    TIMESTAMP_FORMAT = "%Y-%m-%dT%H:%M:%SZ"  # 2009-02-03T17:47:41Z
    RESTRICTED_FIELDS = ["expdate",
                         "cvv2",
                         "acct",
                         ]
    ADMIN_FIELDS = ['id',
                    'user',
                    'flag',
                    'flag_code',
                    'flag_info',
                    'query',
                    'response',
                    'created_at',
                    'updated_at',
                    ]
    ITEM_FIELDS = ["amt",
                   "custom",
                   "invnum",
                   ]
    DIRECT_FIELDS = ["firstname",
                     "lastname",
                     "street",
                     "city",
                     "state",
                     "countrycode",
                     "zip",
                     ]

    # Response fields
    method = models.CharField(max_length=64, blank=True)
    ack = models.CharField(max_length=32, blank=True)
    profilestatus = models.CharField(max_length=32, blank=True)
    timestamp = models.DateTimeField(blank=True, null=True)
    profileid = models.CharField(max_length=32, blank=True)  # I-E596DFUSD882
    profilereference = models.CharField(max_length=128, blank=True)  # PROFILEREFERENCE
    correlationid = models.CharField(max_length=32, blank=True)  # 25b380cda7a21
    token = models.CharField(max_length=64, blank=True)
    payerid = models.CharField(max_length=64, blank=True)

    # Transaction Fields
    firstname = models.CharField("First Name", max_length=255, blank=True)
    lastname = models.CharField("Last Name", max_length=255, blank=True)
    street = models.CharField("Street Address", max_length=255, blank=True)
    city = models.CharField("City", max_length=255, blank=True)
    state = models.CharField("State", max_length=255, blank=True)
    countrycode = models.CharField("Country", max_length=2, blank=True)
    zip = models.CharField("Postal / Zip Code", max_length=32, blank=True)

    # Custom fields
    invnum = models.CharField(max_length=255, blank=True)
    custom = models.CharField(max_length=255, blank=True)

    # Admin fields
    user = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
                             blank=True, null=True,
                             on_delete=models.CASCADE)
    flag = models.BooleanField(default=False, blank=True)
    flag_code = models.CharField(max_length=32, blank=True)
    flag_info = models.TextField(blank=True)
    ipaddress = models.GenericIPAddressField(blank=True, null=True)
    query = models.TextField(blank=True)
    response = models.TextField(blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        db_table = "paypal_nvp"
        verbose_name = "PayPal NVP"

    @cached_property
    def response_dict(self):
        """
        Returns a (MultiValueDict) dictionary containing all the parameters returned in the PayPal response.
        """
        # Undo the urlencode done in init
        return QueryDict(self.response)

    def init(self, request, paypal_request, paypal_response):
        """Initialize a PayPalNVP instance from a HttpRequest."""
        if request is not None:
            from paypal.pro.helpers import strip_ip_port
            self.ipaddress = strip_ip_port(request.META.get('REMOTE_ADDR', ''))
            if (hasattr(request, "user") and request.user.is_authenticated):
                self.user = request.user
        else:
            self.ipaddress = ''

        # No storing credit card info.
        query_data = dict((k, v) for k, v in paypal_request.items() if k not in self.RESTRICTED_FIELDS)
        self.query = urlencode(query_data)
        self.response = urlencode(paypal_response)

        # Was there a flag on the play?
        ack = paypal_response.get('ack', False)
        if ack != "Success":
            if ack == "SuccessWithWarning":
                warn_untested()
                self.flag_info = paypal_response.get('l_longmessage0', '')
            else:
                self.set_flag(paypal_response.get('l_longmessage0', ''), paypal_response.get('l_errorcode', ''))

    def set_flag(self, info, code=None):
        """Flag this instance for investigation."""
        self.flag = True
        self.flag_info += info
        if code is not None:
            self.flag_code = code

    def process(self, request, item):
        """Do a direct payment."""
        warn_untested()
        from paypal.pro.helpers import PayPalWPP

        wpp = PayPalWPP(request)

        # Change the model information into a dict that PayPal can understand.
        params = model_to_dict(self, exclude=self.ADMIN_FIELDS)
        params['acct'] = self.acct
        params['creditcardtype'] = self.creditcardtype
        params['expdate'] = self.expdate
        params['cvv2'] = self.cvv2
        params.update(item)

        # Create recurring payment:
        if 'billingperiod' in params:
            return wpp.createRecurringPaymentsProfile(params, direct=True)
        # Create single payment:
        else:
            return wpp.doDirectPayment(params)

    def __repr__(self):
        return '<PayPalNVP id:{0}>'.format(self.id)

    def __str__(self):
        return "PayPalNVP: {0}".format(self.id)



#!/usr/bin/env python
from __future__ import unicode_literals

import argparse
import sys
import warnings

from django.conf import settings
from django.core.management import execute_from_command_line

warnings.simplefilter("always", PendingDeprecationWarning)
warnings.simplefilter("always", DeprecationWarning)


parser = argparse.ArgumentParser(description="Run the test suite, or specific tests specified using dotted paths.")
parser.add_argument("--use-tz", action='store', default='true',
                    help="Set USE_TZ in settings, 'true'/'false'")

known_args, remaining_args = parser.parse_known_args()

remaining_options = [a for a in remaining_args if a.startswith('-')]
test_args = [a for a in remaining_args if not a.startswith('-')]

settings_dict = dict(
    ROOT_URLCONF='',
    DATABASES={'default': {'ENGINE': 'django.db.backends.sqlite3'}},
    PAYPAL_TEST=True,
    # Please dont make me create another test account and remove this from here :)
    PAYPAL_WPP_USER='django.paypal.seller_api1.gmail.com',
    PAYPAL_WPP_PASSWORD='D5LBCTEKUUJR8EKL',
    PAYPAL_WPP_SIGNATURE='AFcWxV21C7fd0v3bYYYRCpSSRl31A6nHqN3IBok0TF-H8I-C8C6u41Do',
    PAYPAL_IDENTITY_TOKEN='xxx',
    INSTALLED_APPS=[
        'django.contrib.auth',
        'django.contrib.admin',
        'django.contrib.messages',
        'django.contrib.sessions',
        'django.contrib.contenttypes',
        'paypal.pro',
        'paypal.standard',
        'paypal.standard.ipn',
        'paypal.standard.pdt',
    ],
    CACHES={
        'default': {
            'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
            'TIMEOUT': 0,
            'KEY_PREFIX': 'paypal_tests_',
        }
    },
    MIDDLEWARE=[
        "django.contrib.sessions.middleware.SessionMiddleware",
        "django.middleware.common.CommonMiddleware",
        "django.middleware.csrf.CsrfViewMiddleware",
        "django.contrib.auth.middleware.AuthenticationMiddleware",
        "django.contrib.messages.middleware.MessageMiddleware",
        "django.middleware.clickjacking.XFrameOptionsMiddleware",
    ],
    TEMPLATES=[
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': [
            ],
            'APP_DIRS': True,
            'OPTIONS': {
                'context_processors': [
                    'django.contrib.auth.context_processors.auth',
                    'django.template.context_processors.debug',
                    'django.template.context_processors.i18n',
                    'django.template.context_processors.media',
                    'django.template.context_processors.request',
                    'django.template.context_processors.static',
                    'django.template.context_processors.tz',
                    'django.contrib.messages.context_processors.messages',
                ],
            },
        },
    ],
    USE_TZ=not (known_args.use_tz == 'false'),
    LOGGING={
        'version': 1,
        'disable_existing_loggers': True,
        'root': {
            'level': 'WARNING',
            'handlers': ['console'],
        },
        'handlers': {
            'console': {
                'level': 'WARNING',
                'class': 'logging.StreamHandler',
            },
        },
    },
    SECRET_KEY='required_by_django',
    DEFAULT_AUTO_FIELD='django.db.models.BigAutoField',
    )

settings.configure(**settings_dict)


if len(test_args) == 0:
    test_args = ["paypal.pro.tests", "paypal.standard.ipn.tests", "paypal.standard.pdt.tests"]

argv = [sys.argv[0], "test"] + remaining_options + test_args

if __name__ == '__main__':
    execute_from_command_line(argv)



from __future__ import unicode_literals


class PayPalFailure(Exception):
    def __init__(self, msg, nvp=None):
        super(PayPalFailure, self).__init__(msg)
        self.nvp = nvp

    def __str__(self):
        return '{0}, nvp={1}'.format(self.args[0], repr(self.nvp))



from __future__ import unicode_literals

try:
    from django.urls import re_path
except ImportError:
    from django.conf.urls import url as re_path

from django.contrib import admin

from paypal.standard.ipn import views

urlpatterns = [
    re_path(r'^ipn/$', views.ipn),
    re_path(r'^admin/', admin.site.urls),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import hashlib

import six
from django.conf import settings
from django.utils.encoding import smart_str

from paypal.utils import warn_untested


def get_sha1_hexdigest(salt, raw_password):
    encoded_string = (smart_str(salt) + smart_str(raw_password)).encode('utf-8')
    return hashlib.sha1(encoded_string).hexdigest()


def duplicate_txn_id(ipn_obj):
    """
    Returns True if a record with this transaction id exists and its
    payment_status has not changed.
    This function has been completely changed from its previous implementation
    where it used to specifically only check for a Pending->Completed
    transition.

    """

    # get latest similar transaction(s)
    similars = (ipn_obj.__class__._default_manager
                .filter(txn_id=ipn_obj.txn_id)
                .exclude(id=ipn_obj.id)
                .exclude(flag=True)
                .order_by('-created_at')[:1])

    if len(similars) > 0:
        # we have a similar transaction, has the payment_status changed?
        return similars[0].payment_status == ipn_obj.payment_status

    return False


def make_secret(form_instance, secret_fields=None):
    """
    Returns a secret for use in a EWP form or an IPN verification based on a
    selection of variables in params. Should only be used with SSL.

    """
    # @@@ Moved here as temporary fix to avoid dependancy on auth.models.
    # @@@ amount is mc_gross on the IPN - where should mapping logic go?
    # @@@ amount / mc_gross is not nessecarily returned as it was sent - how to use it? 10.00 vs. 10.0
    # @@@ the secret should be based on the invoice or custom fields as well - otherwise its always the same.

    # Build the secret with fields availible in both PaymentForm and the IPN. Order matters.
    if secret_fields is None:
        secret_fields = ['business', 'item_name']

    data = ""
    for name in secret_fields:
        if hasattr(form_instance, 'cleaned_data'):
            if name in form_instance.cleaned_data:
                data += six.text_type(form_instance.cleaned_data[name])
        else:
            # Initial data passed into the constructor overrides defaults.
            if name in form_instance.initial:
                data += six.text_type(form_instance.initial[name])
            elif name in form_instance.fields and form_instance.fields[name].initial is not None:
                data += six.text_type(form_instance.fields[name].initial)

    secret = get_sha1_hexdigest(settings.SECRET_KEY, data)
    return secret


def check_secret(form_instance, secret):
    """
    Returns true if received `secret` matches expected secret for form_instance.
    Used to verify IPN.

    """
    warn_untested()
    # @@@ add invoice & custom
    # secret_fields = ['business', 'item_name']
    return make_secret(form_instance) == secret



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PayPalPDT',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('business', models.CharField(help_text='Email where the money was sent.', max_length=127, blank=True)),
                ('charset', models.CharField(max_length=32, blank=True)),
                ('custom', models.CharField(max_length=255, blank=True)),
                ('notify_version', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('parent_txn_id', models.CharField(max_length=19, verbose_name='Parent Transaction ID', blank=True)),
                ('receiver_email', models.EmailField(max_length=127, blank=True)),
                ('receiver_id', models.CharField(max_length=127, blank=True)),
                ('residence_country', models.CharField(max_length=2, blank=True)),
                ('test_ipn', models.BooleanField(default=False)),
                ('txn_id', models.CharField(help_text='PayPal transaction ID.', max_length=19, verbose_name='Transaction ID', db_index=True, blank=True)),
                ('txn_type', models.CharField(help_text='PayPal transaction type.', max_length=128, verbose_name='Transaction Type', blank=True)),
                ('verify_sign', models.CharField(max_length=255, blank=True)),
                ('address_country', models.CharField(max_length=64, blank=True)),
                ('address_city', models.CharField(max_length=40, blank=True)),
                ('address_country_code', models.CharField(help_text='ISO 3166', max_length=64, blank=True)),
                ('address_name', models.CharField(max_length=128, blank=True)),
                ('address_state', models.CharField(max_length=40, blank=True)),
                ('address_status', models.CharField(max_length=11, blank=True)),
                ('address_street', models.CharField(max_length=200, blank=True)),
                ('address_zip', models.CharField(max_length=20, blank=True)),
                ('contact_phone', models.CharField(max_length=20, blank=True)),
                ('first_name', models.CharField(max_length=64, blank=True)),
                ('last_name', models.CharField(max_length=64, blank=True)),
                ('payer_business_name', models.CharField(max_length=127, blank=True)),
                ('payer_email', models.CharField(max_length=127, blank=True)),
                ('payer_id', models.CharField(max_length=13, blank=True)),
                ('auth_amount', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('auth_exp', models.CharField(max_length=28, blank=True)),
                ('auth_id', models.CharField(max_length=19, blank=True)),
                ('auth_status', models.CharField(max_length=9, blank=True)),
                ('exchange_rate', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=16, blank=True)),
                ('invoice', models.CharField(max_length=127, blank=True)),
                ('item_name', models.CharField(max_length=127, blank=True)),
                ('item_number', models.CharField(max_length=127, blank=True)),
                ('mc_currency', models.CharField(default='USD', max_length=32, blank=True)),
                ('mc_fee', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('mc_gross', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('mc_handling', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('mc_shipping', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('memo', models.CharField(max_length=255, blank=True)),
                ('num_cart_items', models.IntegerField(default=0, null=True, blank=True)),
                ('option_name1', models.CharField(max_length=64, blank=True)),
                ('option_name2', models.CharField(max_length=64, blank=True)),
                ('payer_status', models.CharField(max_length=10, blank=True)),
                ('payment_date', models.DateTimeField(help_text='HH:MM:SS DD Mmm YY, YYYY PST', null=True, blank=True)),
                ('payment_gross', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('payment_status', models.CharField(max_length=17, blank=True)),
                ('payment_type', models.CharField(max_length=7, blank=True)),
                ('pending_reason', models.CharField(max_length=14, blank=True)),
                ('protection_eligibility', models.CharField(max_length=32, blank=True)),
                ('quantity', models.IntegerField(default=1, null=True, blank=True)),
                ('reason_code', models.CharField(max_length=15, blank=True)),
                ('remaining_settle', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('settle_amount', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('settle_currency', models.CharField(max_length=32, blank=True)),
                ('shipping', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('shipping_method', models.CharField(max_length=255, blank=True)),
                ('tax', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('transaction_entity', models.CharField(max_length=7, blank=True)),
                ('auction_buyer_id', models.CharField(max_length=64, blank=True)),
                ('auction_closing_date', models.DateTimeField(help_text='HH:MM:SS DD Mmm YY, YYYY PST', null=True, blank=True)),
                ('auction_multi_item', models.IntegerField(default=0, null=True, blank=True)),
                ('for_auction', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('amount', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('amount_per_cycle', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('initial_payment_amount', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('next_payment_date', models.DateTimeField(help_text='HH:MM:SS DD Mmm YY, YYYY PST', null=True, blank=True)),
                ('outstanding_balance', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('payment_cycle', models.CharField(max_length=32, blank=True)),
                ('period_type', models.CharField(max_length=32, blank=True)),
                ('product_name', models.CharField(max_length=128, blank=True)),
                ('product_type', models.CharField(max_length=128, blank=True)),
                ('profile_status', models.CharField(max_length=32, blank=True)),
                ('recurring_payment_id', models.CharField(max_length=128, blank=True)),
                ('rp_invoice_id', models.CharField(max_length=127, blank=True)),
                ('time_created', models.DateTimeField(help_text='HH:MM:SS DD Mmm YY, YYYY PST', null=True, blank=True)),
                ('amount1', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('amount2', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('amount3', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('mc_amount1', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('mc_amount2', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('mc_amount3', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('password', models.CharField(max_length=24, blank=True)),
                ('period1', models.CharField(max_length=32, blank=True)),
                ('period2', models.CharField(max_length=32, blank=True)),
                ('period3', models.CharField(max_length=32, blank=True)),
                ('reattempt', models.CharField(max_length=1, blank=True)),
                ('recur_times', models.IntegerField(default=0, null=True, blank=True)),
                ('recurring', models.CharField(max_length=1, blank=True)),
                ('retry_at', models.DateTimeField(help_text='HH:MM:SS DD Mmm YY, YYYY PST', null=True, blank=True)),
                ('subscr_date', models.DateTimeField(help_text='HH:MM:SS DD Mmm YY, YYYY PST', null=True, blank=True)),
                ('subscr_effective', models.DateTimeField(help_text='HH:MM:SS DD Mmm YY, YYYY PST', null=True, blank=True)),
                ('subscr_id', models.CharField(max_length=19, blank=True)),
                ('username', models.CharField(max_length=64, blank=True)),
                ('case_creation_date', models.DateTimeField(help_text='HH:MM:SS DD Mmm YY, YYYY PST', null=True, blank=True)),
                ('case_id', models.CharField(max_length=14, blank=True)),
                ('case_type', models.CharField(max_length=24, blank=True)),
                ('receipt_id', models.CharField(max_length=64, blank=True)),
                ('currency_code', models.CharField(default='USD', max_length=32, blank=True)),
                ('handling_amount', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('transaction_subject', models.CharField(max_length=255, blank=True)),
                ('ipaddress', models.IPAddressField(blank=True)),
                ('flag', models.BooleanField(default=False)),
                ('flag_code', models.CharField(max_length=16, blank=True)),
                ('flag_info', models.TextField(blank=True)),
                ('query', models.TextField(blank=True)),
                ('response', models.TextField(blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('from_view', models.CharField(max_length=6, null=True, blank=True)),
                ('amt', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
                ('cm', models.CharField(max_length=255, blank=True)),
                ('sig', models.CharField(max_length=255, blank=True)),
                ('tx', models.CharField(max_length=255, blank=True)),
                ('st', models.CharField(max_length=32, blank=True)),
            ],
            options={
                'db_table': 'paypal_pdt',
                'verbose_name': 'PayPal PDT',
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pro', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paypalnvp',
            name='ipaddress',
            field=models.GenericIPAddressField(null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PayPalNVP',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('method', models.CharField(max_length=64, blank=True)),
                ('ack', models.CharField(max_length=32, blank=True)),
                ('profilestatus', models.CharField(max_length=32, blank=True)),
                ('timestamp', models.DateTimeField(null=True, blank=True)),
                ('profileid', models.CharField(max_length=32, blank=True)),
                ('profilereference', models.CharField(max_length=128, blank=True)),
                ('correlationid', models.CharField(max_length=32, blank=True)),
                ('token', models.CharField(max_length=64, blank=True)),
                ('payerid', models.CharField(max_length=64, blank=True)),
                ('firstname', models.CharField(max_length=255, verbose_name='First Name', blank=True)),
                ('lastname', models.CharField(max_length=255, verbose_name='Last Name', blank=True)),
                ('street', models.CharField(max_length=255, verbose_name='Street Address', blank=True)),
                ('city', models.CharField(max_length=255, verbose_name='City', blank=True)),
                ('state', models.CharField(max_length=255, verbose_name='State', blank=True)),
                ('countrycode', models.CharField(max_length=2, verbose_name='Country', blank=True)),
                ('zip', models.CharField(max_length=32, verbose_name='Postal / Zip Code', blank=True)),
                ('invnum', models.CharField(max_length=255, blank=True)),
                ('custom', models.CharField(max_length=255, blank=True)),
                ('flag', models.BooleanField(default=False)),
                ('flag_code', models.CharField(max_length=32, blank=True)),
                ('flag_info', models.TextField(blank=True)),
                ('ipaddress', models.IPAddressField(blank=True)),
                ('query', models.TextField(blank=True)),
                ('response', models.TextField(blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('user', models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True,
                                           on_delete=models.CASCADE)),
            ],
            options={
                'db_table': 'paypal_nvp',
                'verbose_name': 'PayPal NVP',
            },
            bases=(models.Model,),
        ),
    ]



#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from decimal import Decimal

try:
    from unittest import mock
except ImportError:
    import mock
from django.contrib.auth import get_user_model
from django.forms import ValidationError
from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings
from vcr import VCR

from paypal.pro.exceptions import PayPalFailure
from paypal.pro.fields import CreditCardField
from paypal.pro.helpers import VERSION, PayPalError, PayPalWPP, strip_ip_port
from paypal.pro.views import PayPalPro

from .settings import TEMPLATES

RF = RequestFactory()

vcr = VCR(path_transformer=VCR.ensure_suffix('.yaml'))


def make_request(user=None):
    request = RF.get("/pay/", REMOTE_ADDR="127.0.0.1:8000")
    if user is not None:
        request.user = user
    return request


class CreditCardFieldTest(TestCase):
    def test_CreditCardField(self):
        field = CreditCardField()
        field.clean('4797503429879309')
        self.assertEqual(field.card_type, "Visa")
        self.assertRaises(ValidationError, CreditCardField().clean, '1234567890123455')

    def test_invalidCreditCards(self):
        self.assertEqual(CreditCardField().clean('4797-5034-2987-9309'), '4797503429879309')


def ppp_wrapper(request, handler=None):
    item = {"paymentrequest_0_amt": "10.00",
            "inv": "inventory",
            "custom": "tracking",
            "cancelurl": "http://foo.com/cancel",
            "returnurl": "http://foo.com/return"}

    if handler is None:
        handler = lambda nvp: nvp  # NOP
    ppp = PayPalPro(
        item=item,                             # what you're selling
        payment_template="payment.html",       # template name for payment
        confirm_template="confirmation.html",  # template name for confirmation
        success_url="/success/",               # redirect location after success
        nvp_handler=handler
        )

    return ppp(request)


@override_settings(TEMPLATES=TEMPLATES)
class PayPalProTest(TestCase):

    @vcr.use_cassette()
    def test_get(self):
        response = ppp_wrapper(RF.get('/'))
        self.assertContains(response, 'Show me the money')
        self.assertEqual(response.status_code, 200)

    @vcr.use_cassette()
    def test_get_redirect(self):
        response = ppp_wrapper(RF.get('/', {'express': '1'}))
        self.assertEqual(response.status_code, 302)

    @vcr.use_cassette()
    def test_validate_confirm_form_error(self):
        response = ppp_wrapper(RF.post('/',
                                       {'token': '123',
                                        'PayerID': '456'}))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context_data.get('errors', ''),
                         PayPalPro.errors['processing'])

    @vcr.use_cassette()
    @mock.patch.object(PayPalWPP, 'doExpressCheckoutPayment', autospec=True)
    def test_validate_confirm_form_ok(self, doExpressCheckoutPayment):
        nvp = {'mock': True}
        doExpressCheckoutPayment.return_value = nvp

        received = []

        def handler(nvp):
            received.append(nvp)

        response = ppp_wrapper(RF.post('/',
                                       {'token': '123',
                                        'PayerID': '456'}),
                               handler=handler)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], '/success/')
        self.assertEqual(len(received), 1)


class PayPalWPPTest(TestCase):
    def setUp(self):

        self.item = {
            'amt': '9.95',
            'inv': 'inv',
            'custom': 'custom',
            'next': 'http://www.example.com/next/',
            'returnurl': 'http://www.example.com/pay/',
            'cancelurl': 'http://www.example.com/cancel/'
        }
        # Handle different parameters for Express Checkout
        self.ec_item = {
            'paymentrequest_0_amt': '9.95',
            'inv': 'inv',
            'custom': 'custom',
            'next': 'http://www.example.com/next/',
            'returnurl': 'http://www.example.com/pay/',
            'cancelurl': 'http://www.example.com/cancel/'
        }

    def get_valid_doDirectPayment_data(self):
        return {
            'firstname': 'Brave',
            'lastname': 'Star',
            'street': '1 Main St',
            'city': u'San Jos\xe9',
            'state': 'CA',
            'countrycode': 'US',
            'zip': '95131',
            'acct': '4032039938039650',
            'expdate': '112021',
            'cvv2': '',
            'creditcardtype': 'visa',
            'ipaddress': '10.0.1.199',
        }

    @vcr.use_cassette()
    def test_doDirectPayment_missing_params(self):
        wpp = PayPalWPP(make_request())
        data = {'firstname': 'Chewbacca'}
        self.assertRaises(PayPalError, wpp.doDirectPayment, data)

    @vcr.use_cassette()
    def test_doDirectPayment_valid(self):
        wpp = PayPalWPP(make_request())
        data = self.get_valid_doDirectPayment_data()
        data.update(self.item)
        nvp = wpp.doDirectPayment(data)
        self.assertIsNotNone(nvp)
        for k, v in [('avscode', 'X'),
                     ('amt', '9.95'),
                     ('correlationid', '1025431f33d89'),
                     ('currencycode', 'USD'),
                     ('ack', 'Success')]:
            self.assertEqual(nvp.response_dict[k], v)

    @vcr.use_cassette()
    def test_doDirectPayment_authenticated_user(self):
        User = get_user_model()
        user = User.objects.create(username='testuser')
        wpp = PayPalWPP(make_request(user=user))
        data = self.get_valid_doDirectPayment_data()
        data.update(self.item)
        npm_obj = wpp.doDirectPayment(data)
        self.assertEqual(npm_obj.user, user)

    @vcr.use_cassette()
    def test_doDirectPayment_invalid(self):
        wpp = PayPalWPP(make_request())
        data = {
            'firstname': 'Epic',
            'lastname': 'Fail',
            'street': '100 Georgia St',
            'city': 'Vancouver',
            'state': 'BC',
            'countrycode': 'CA',
            'zip': 'V6V 1V1',
            'expdate': '012019',
            'cvv2': '999',
            'acct': '1234567890',
            'creditcardtype': 'visa',
            'ipaddress': '10.0.1.199', }
        data.update(self.item)
        self.assertRaises(PayPalFailure, wpp.doDirectPayment, data)

    @vcr.use_cassette()
    def test_setExpressCheckout(self):
        wpp = PayPalWPP(make_request())
        nvp_obj = wpp.setExpressCheckout(self.ec_item)
        self.assertEqual(nvp_obj.ack, "Success")

    @vcr.use_cassette()
    @mock.patch.object(PayPalWPP, '_request', autospec=True)
    def test_doExpressCheckoutPayment(self, mock_request_object):
        ec_token = 'EC-1234567890'
        payerid = 'LXYZABC1234'
        item = self.ec_item.copy()
        item.update({'token': ec_token, 'payerid': payerid})
        mock_request_object.return_value = 'ack=Success&token=%s&version=%spaymentinfo_0_amt=%s' % \
            (ec_token, VERSION, self.ec_item['paymentrequest_0_amt'])
        wpp = PayPalWPP(make_request())
        wpp.doExpressCheckoutPayment(item)
        call_args = mock_request_object.call_args
        self.assertIn('VERSION=%s' % VERSION, call_args[0][1])
        self.assertIn('METHOD=DoExpressCheckoutPayment', call_args[0][1])
        self.assertIn('TOKEN=%s' % ec_token, call_args[0][1])
        self.assertIn('PAYMENTREQUEST_0_AMT=%s' % item['paymentrequest_0_amt'],
                      call_args[0][1])
        self.assertIn('PAYERID=%s' % payerid, call_args[0][1])

    @vcr.use_cassette()
    @mock.patch.object(PayPalWPP, '_request', autospec=True)
    def test_doExpressCheckoutPayment_invalid(self, mock_request_object):
        ec_token = 'EC-1234567890'
        payerid = 'LXYZABC1234'
        item = self.ec_item.copy()
        item.update({'token': ec_token, 'payerid': payerid})
        mock_request_object.return_value = 'ack=Failure&l_errorcode=42&l_longmessage0=Broken'
        wpp = PayPalWPP(make_request())
        with self.assertRaises(PayPalFailure):
            wpp.doExpressCheckoutPayment(item)

    @vcr.use_cassette()
    @mock.patch.object(PayPalWPP, '_request', autospec=True)
    def test_createBillingAgreement(self, mock_request_object):
        mock_request_object.return_value = 'ack=Success&billingagreementid=B-XXXXX&version=%s' % VERSION
        wpp = PayPalWPP(make_request())
        nvp = wpp.createBillingAgreement({'token': 'dummy token'})
        call_args = mock_request_object.call_args
        self.assertIn('VERSION=%s' % VERSION, call_args[0][1])
        self.assertIn('METHOD=CreateBillingAgreement', call_args[0][1])
        self.assertIn('TOKEN=dummy+token', call_args[0][1])
        self.assertEqual(nvp.method, 'CreateBillingAgreement')
        self.assertEqual(nvp.ack, 'Success')
        mock_request_object.return_value = 'ack=Failure&l_errorcode=42&l_longmessage0=Broken'
        with self.assertRaises(PayPalFailure):
            nvp = wpp.createBillingAgreement({'token': 'dummy token'})

    @vcr.use_cassette()
    @mock.patch.object(PayPalWPP, '_request', autospec=True)
    def test_doReferenceTransaction_valid(self, mock_request_object):
        reference_id = 'B-1234'
        amount = Decimal('10.50')
        mock_request_object.return_value = (
            'ack=Success&paymentstatus=Completed&amt=%s&version=%s&billingagreementid=%s' %
            (amount, VERSION, reference_id))
        wpp = PayPalWPP(make_request())
        nvp = wpp.doReferenceTransaction({'referenceid': reference_id,
                                          'amt': amount})
        call_args = mock_request_object.call_args
        self.assertIn('VERSION=%s' % VERSION, call_args[0][1])
        self.assertIn('METHOD=DoReferenceTransaction', call_args[0][1])
        self.assertIn('REFERENCEID=%s' % reference_id, call_args[0][1])
        self.assertIn('AMT=%s' % amount, call_args[0][1])
        self.assertEqual(nvp.method, 'DoReferenceTransaction')
        self.assertEqual(nvp.ack, 'Success')

    @vcr.use_cassette()
    @mock.patch.object(PayPalWPP, '_request', autospec=True)
    def test_doReferenceTransaction_invalid(self, mock_request_object):
        reference_id = 'B-1234'
        amount = Decimal('10.50')
        mock_request_object.return_value = 'ack=Failure&l_errorcode=42&l_longmessage0=Broken'
        wpp = PayPalWPP(make_request())
        with self.assertRaises(PayPalFailure):
            wpp.doReferenceTransaction({'referenceid': reference_id,
                                        'amt': amount})

    def test_strip_ip_port(self):
        IPv4 = '192.168.0.1'
        IPv6 = '2001:0db8:85a3:0000:0000:8a2e:0370:7334'
        PORT = '8000'

        # IPv4 with port
        test = '%s:%s' % (IPv4, PORT)
        self.assertEqual(IPv4, strip_ip_port(test))

        # IPv4 without port
        test = IPv4
        self.assertEqual(IPv4, strip_ip_port(test))

        # IPv6 with port
        test = '[%s]:%s' % (IPv6, PORT)
        self.assertEqual(IPv6, strip_ip_port(test))

        # IPv6 without port
        test = IPv6
        self.assertEqual(IPv6, strip_ip_port(test))

        # No IP
        self.assertEqual('', strip_ip_port(''))

# -- DoExpressCheckoutPayment
# PayPal Request:
# {'amt': '10.00',
#  'cancelurl': u'http://xxx.xxx.xxx.xxx/deploy/480/upgrade/?upgrade=cname',
#  'custom': u'website_id=480&cname=1',
#  'inv': u'website-480-cname',
#  'method': 'DoExpressCheckoutPayment',
#  'next': u'http://xxx.xxx.xxx.xxx/deploy/480/upgrade/?upgrade=cname',
#  'payerid': u'BN5JZ2V7MLEV4',
#  'paymentaction': 'Sale',
#  'returnurl': u'http://xxx.xxx.xxx.xxx/deploy/480/upgrade/?upgrade=cname',
#  'token': u'EC-6HW17184NE0084127'}
#
# PayPal Response:
# {'ack': 'Success',
#  'amt': '10.00',
#  'build': '848077',
#  'correlationid': '375f4773c3d34',
#  'currencycode': 'USD',
#  'feeamt': '0.59',
#  'ordertime': '2009-03-04T20:56:08Z',
#  'paymentstatus': 'Completed',
#  'paymenttype': 'instant',
#  'pendingreason': 'None',
#  'reasoncode': 'None',
#  'taxamt': '0.00',
#  'timestamp': '2009-03-04T20:56:09Z',
#  'token': 'EC-6HW17184NE0084127',
#  'transactionid': '3TG42202A7335864V',
#  'transactiontype': 'expresscheckout',
#  'version': '54.0'}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.contrib import admin, messages

from paypal.standard.ipn.models import PayPalIPN


def reverify_flagged(modeladmin, request, queryset):
    q = queryset.filter(flag=True)
    for ipn in q:
        ipn.verify()
        ipn.send_signals()
    messages.info(request, "{0} IPN object(s) re-verified".format(len(q)))


reverify_flagged.short_description = "Re-verify selected flagged IPNs"


class PayPalIPNAdmin(admin.ModelAdmin):
    list_filter = [
        'payment_status',
        'flag',
        'txn_type',
    ]
    date_hierarchy = 'payment_date'
    fieldsets = (
        (None, {
            "fields": [
                "flag", "txn_id", "txn_type", "payment_status", "payment_date",
                "transaction_entity", "reason_code", "pending_reason",
                "mc_currency", "mc_gross", "mc_fee", "mc_handling", "mc_shipping",
                "auth_status", "auth_amount", "auth_exp", "auth_id"
            ]
        }),
        ("Address", {
            "description": "The address of the Buyer.",
            'classes': ('collapse',),
            "fields": [
                "address_city", "address_country", "address_country_code",
                "address_name", "address_state", "address_status",
                "address_street", "address_zip"
            ]
        }),
        ("Buyer", {
            "description": "The information about the Buyer.",
            'classes': ('collapse',),
            "fields": [
                "first_name", "last_name", "payer_business_name", "payer_email",
                "payer_id", "payer_status", "contact_phone", "residence_country"
            ]
        }),
        ("Seller", {
            "description": "The information about the Seller.",
            'classes': ('collapse',),
            "fields": [
                "business", "item_name", "item_number", "quantity",
                "receiver_email", "receiver_id", "custom", "invoice", "memo"
            ]
        }),
        ("Recurring", {
            "description": "Information about recurring Payments.",
            "classes": ("collapse",),
            "fields": [
                "profile_status", "initial_payment_amount", "amount_per_cycle",
                "outstanding_balance", "period_type", "product_name",
                "product_type", "recurring_payment_id", "receipt_id",
                "next_payment_date"
            ]
        }),
        ("Subscription", {
            "description": "Information about recurring Subscptions.",
            "classes": ("collapse",),
            "fields": [
                "subscr_date", "subscr_effective", "period1", "period2",
                "period3", "amount1", "amount2", "amount3", "mc_amount1",
                "mc_amount2", "mc_amount3", "recurring", "reattempt",
                "retry_at", "recur_times", "username", "password", "subscr_id"
            ]
        }),
        ("Admin", {
            "description": "Additional Info.",
            "classes": ('collapse',),
            "fields": [
                "test_ipn", "ipaddress", "query", "response", "flag_code",
                "flag_info"
            ]
        }),
    )
    list_display = [
        "__unicode__", "flag", "flag_info", "invoice", "custom",
        "payment_status", "created_at"
    ]
    search_fields = ["txn_id", "recurring_payment_id", "subscr_id"]

    actions = [reverify_flagged]


admin.site.register(PayPalIPN, PayPalIPNAdmin)



"""
run this with ./manage.py test website
see http://www.djangoproject.com/documentation/testing/ for details
"""
from __future__ import unicode_literals

try:
    from unittest import mock
except ImportError:
    import mock
from django.conf import settings
from django.template.loader import render_to_string
from django.test import TestCase
from django.test.utils import override_settings

from paypal.standard.conf import POSTBACK_ENDPOINT, SANDBOX_POSTBACK_ENDPOINT
from paypal.standard.pdt.models import PayPalPDT

from .settings import TEMPLATES


class DummyPayPalPDT(object):
    def __init__(self, update_context_dict={}):
        self.context_dict = {'st': 'SUCCESS', 'custom': 'cb736658-3aad-4694-956f-d0aeade80194',
                             'txn_id': '1ED550410S3402306', 'mc_gross': '225.00',
                             'business': 'test@example.com', 'error': 'Error code: 1234'}

        self.context_dict.update(update_context_dict)
        self.response = ''

    def update_with_get_params(self, get_params):
        if 'tx' in get_params:
            self.context_dict['txn_id'] = get_params.get('tx')
        if 'amt' in get_params:
            self.context_dict['mc_gross'] = get_params.get('amt')
        if 'cm' in get_params:
            self.context_dict['custom'] = get_params.get('cm')

    def _postback(self, test=True):
        """Perform a Fake PayPal PDT Postback request."""
        return render_to_string("pdt/test_pdt_response.html", self.context_dict).encode('utf-8')


@override_settings(ROOT_URLCONF="paypal.standard.pdt.tests.test_urls",
                   TEMPLATES=TEMPLATES)
class PDTTest(TestCase):

    def setUp(self):
        # set up some dummy PDT get parameters
        self.get_params = {"tx": "4WJ86550014687441", "st": "Completed", "amt": "225.00", "cc": "EUR",
                           "cm": "a3e192b8-8fea-4a86-b2e8-d5bf502e36be", "item_number": "",
                           "sig": "blahblahblah"}

        # monkey patch the PayPalPDT._postback function
        self.dpppdt = DummyPayPalPDT()
        self.dpppdt.update_with_get_params(self.get_params)
        PayPalPDT._postback = self.dpppdt._postback

    def test_verify_postback(self):
        dpppdt = DummyPayPalPDT()
        paypal_response = dpppdt._postback().decode('ascii')
        assert ('SUCCESS' in paypal_response)
        self.assertEqual(len(PayPalPDT.objects.all()), 0)
        pdt_obj = PayPalPDT()
        pdt_obj.ipaddress = '127.0.0.1'
        pdt_obj.response = paypal_response
        pdt_obj._verify_postback()
        self.assertEqual(len(PayPalPDT.objects.all()), 0)
        self.assertEqual(pdt_obj.txn_id, '1ED550410S3402306')
        # Issue #121: Ensure for doesn't blank non-PayPal-supplied fields
        self.assertEqual(pdt_obj.ipaddress, '127.0.0.1')
        self.assertEqual(pdt_obj.response, paypal_response)

    def test_pdt(self):
        self.assertEqual(len(PayPalPDT.objects.all()), 0)
        self.dpppdt.update_with_get_params(self.get_params)
        paypal_response = self.client.get("/pdt/", self.get_params)
        self.assertContains(paypal_response, 'Transaction complete', status_code=200)
        self.assertEqual(len(PayPalPDT.objects.all()), 1)

    def test_double_pdt_get(self):
        self.assertEqual(len(PayPalPDT.objects.all()), 0)
        paypal_response = self.client.get("/pdt/", self.get_params)
        self.assertContains(paypal_response, 'Transaction complete', status_code=200)
        self.assertEqual(len(PayPalPDT.objects.all()), 1)
        pdt_obj = PayPalPDT.objects.all()[0]
        self.assertEqual(pdt_obj.flag, False)
        paypal_response = self.client.get("/pdt/", self.get_params)
        self.assertContains(paypal_response, 'Transaction complete', status_code=200)
        self.assertEqual(len(PayPalPDT.objects.all()), 1)  # we don't create a new pdt
        pdt_obj = PayPalPDT.objects.all()[0]
        self.assertEqual(pdt_obj.flag, False)

    def test_no_txn_id_in_pdt(self):
        self.dpppdt.context_dict.pop('txn_id')
        self.get_params = {}
        paypal_response = self.client.get("/pdt/", self.get_params)
        self.assertContains(paypal_response, 'Transaction Failed', status_code=200)
        self.assertEqual(len(PayPalPDT.objects.all()), 0)

    def test_custom_passthrough(self):
        self.assertEqual(len(PayPalPDT.objects.all()), 0)
        self.dpppdt.update_with_get_params(self.get_params)
        paypal_response = self.client.get("/pdt/", self.get_params)
        self.assertContains(paypal_response, 'Transaction complete', status_code=200)
        self.assertEqual(len(PayPalPDT.objects.all()), 1)
        pdt_obj = PayPalPDT.objects.all()[0]
        self.assertEqual(pdt_obj.custom, self.get_params['cm'])


class MockedResponse:
    content = 'test'


def mocked_post(*args, **kwargs):
    url = args[0]
    data = kwargs['data']

    assert url == SANDBOX_POSTBACK_ENDPOINT
    assert data['cmd'] == '_notify-synch'
    assert 'at' in data
    assert 'tx' in data

    return MockedResponse()


class PDTPostbackTest(TestCase):
    """
    Tests an actual postback to PayPal server.
    """
    @classmethod
    def setUpClass(cls):
        cls.pdt = PayPalPDT()

    @classmethod
    def tearDownClass(cls):
        pass

    @mock.patch('paypal.standard.pdt.models.requests.post', side_effect=mocked_post)
    def test_postback(self, post):
        response = self.pdt._postback()
        self.assertEqual(response, MockedResponse.content)

    def test_enpoint(self):
        endpoint = self.pdt.get_endpoint()

        if getattr(settings, 'PAYPAL_TEST', True):
            self.assertEqual(endpoint, SANDBOX_POSTBACK_ENDPOINT)
        else:
            self.assertEqual(endpoint, POSTBACK_ENDPOINT)



#!/usr/bin/env python

# This manage.py exists for the purpose of creating migrations
import sys

from django.conf import settings
from django.core.management import execute_from_command_line

settings.configure(
    ROOT_URLCONF='paypal.standard.ipn.tests.test_urls',
    DATABASES={'default':
               {'ENGINE': 'django.db.backends.sqlite3',
                'NAME': 'test.db',
                }},
    PAYPAL_IDENTITY_TOKEN='',
    INSTALLED_APPS=[
        'django.contrib.auth',
        'django.contrib.admin',
        'django.contrib.contenttypes',
        'paypal.pro',
        'paypal.standard',
        'paypal.standard.ipn',
        'paypal.standard.pdt',
    ],
    CACHES={
        'default': {
            'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
            'TIMEOUT': 0,
            'KEY_PREFIX': 'paypal_tests_',
        }
    },
    MIDDLEWARE_CLASSES=[],
)

if __name__ == '__main__':
    execute_from_command_line(sys.argv)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from paypal.standard.forms import PayPalStandardBaseForm
from paypal.standard.pdt.models import PayPalPDT


class PayPalPDTForm(PayPalStandardBaseForm):
    class Meta:
        model = PayPalPDT
        exclude = ['ipaddress', 'flag', 'flag_code',
                   'flag_info', 'query', 'response',
                   'created_at', 'updated', 'form_view']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging

from django.http import HttpResponse, QueryDict
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

from paypal.standard.ipn.forms import PayPalIPNForm
from paypal.standard.ipn.models import PayPalIPN
from paypal.standard.models import DEFAULT_ENCODING
from paypal.utils import warn_untested

logger = logging.getLogger(__name__)

CONTENT_TYPE_ERROR = ("Invalid Content-Type - PayPal is only expected to use "
                      "application/x-www-form-urlencoded. If using django's "
                      "test Client, set `content_type` explicitly")


@require_POST
@csrf_exempt
def ipn(request):
    """
    PayPal IPN endpoint (notify_url).
    Used by both PayPal Payments Pro and Payments Standard to confirm transactions.
    http://tinyurl.com/d9vu9d

    PayPal IPN Simulator:
    https://developer.paypal.com/cgi-bin/devscr?cmd=_ipn-link-session
    """
    # TODO: Clean up code so that we don't need to set None here and have a lot
    #       of if checks just to determine if flag is set.
    flag = None
    ipn_obj = None

    # Avoid the RawPostDataException. See original issue for details:
    # https://github.com/spookylukey/django-paypal/issues/79
    if not request.META.get('CONTENT_TYPE', '').startswith(
            'application/x-www-form-urlencoded'):
        raise AssertionError(CONTENT_TYPE_ERROR)

    logger.debug("PayPal incoming POST data: %s", request.body)

    # Clean up the data as PayPal sends some weird values such as "N/A"
    # Also, need to cope with custom encoding, which is stored in the body (!).
    # Assuming the tolerant parsing of QueryDict and an ASCII-like encoding,
    # such as windows-1252, latin1 or UTF8, the following will work:
    encoding = request.POST.get('charset', None)

    encoding_missing = encoding is None
    if encoding_missing:
        encoding = DEFAULT_ENCODING

    try:
        data = QueryDict(request.body, encoding=encoding).copy()
    except LookupError:
        warn_untested()
        data = None
        flag = "Invalid form - invalid charset"

    if data is not None:
        if hasattr(PayPalIPN._meta, 'get_fields'):
            date_fields = [f.attname for f in PayPalIPN._meta.get_fields() if f.__class__.__name__ == 'DateTimeField']
        else:
            date_fields = [f.attname for f, m in PayPalIPN._meta.get_fields_with_model()
                           if f.__class__.__name__ == 'DateTimeField']

        for date_field in date_fields:
            if data.get(date_field) == 'N/A':
                del data[date_field]

        form = PayPalIPNForm(data)
        if form.is_valid():
            try:
                # When commit = False, object is returned without saving to DB.
                ipn_obj = form.save(commit=False)
            except Exception as e:
                flag = "Exception while processing. (%s)" % e
        else:
            formatted_form_errors = ["{0}: {1}".format(k, ", ".join(v)) for k, v in form.errors.items()]
            flag = "Invalid form. ({0})".format(", ".join(formatted_form_errors))

    if ipn_obj is None:
        ipn_obj = PayPalIPN()

    # Set query params and sender's IP address
    ipn_obj.initialize(request)

    if flag is not None:
        # We save errors in the flag field
        ipn_obj.set_flag(flag)
    else:
        # Secrets should only be used over SSL.
        if request.is_secure() and 'secret' in request.GET:
            warn_untested()
            ipn_obj.verify_secret(form, request.GET['secret'])
        else:
            ipn_obj.verify()

    ipn_obj.save()
    ipn_obj.send_signals()

    if encoding_missing:
        # Wait until we have an ID to log warning
        logger.warning("No charset passed with PayPalIPN: %s. Guessing %s", ipn_obj.id, encoding)

    return HttpResponse("OKAY")



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.test import TestCase
from django.test.utils import override_settings
from django.urls import reverse

from paypal.standard.ipn.models import PayPalIPN
from paypal.standard.ipn.signals import valid_ipn_received

from .test_ipn import IPN_POST_PARAMS, IPNUtilsMixin, MockedPostbackMixin


@override_settings(ROOT_URLCONF='paypal.standard.ipn.tests.test_urls')
class AdminTest(MockedPostbackMixin, IPNUtilsMixin, TestCase):
    def setUp(self):
        super(AdminTest, self).setUp()
        User = get_user_model()
        user = User.objects.create_superuser(username="admin",
                                             email="admin@example.com",
                                             password="password")
        self.user = user

    def test_verify_action(self):
        PayPalIPN._postback = lambda self: b"Internal Server Error"
        self.paypal_post(IPN_POST_PARAMS)
        ipn_obj = PayPalIPN.objects.get()
        self.assertEqual(ipn_obj.flag, True)

        url = reverse('admin:ipn_paypalipn_changelist')
        self.assertTrue(self.client.login(username='admin',
                                          password='password'))
        response = self.client.get(url)
        self.assertContains(response, IPN_POST_PARAMS['txn_id'])

        self.got_signal = False
        self.signal_obj = None

        def handle_signal(sender, **kwargs):
            self.got_signal = True
            self.signal_obj = sender

        valid_ipn_received.connect(handle_signal)

        PayPalIPN._postback = lambda self: b"VERIFIED"
        response_2 = self.client.post(url,
                                      {'action': 'reverify_flagged',
                                       '_selected_action': [str(ipn_obj.id)]})
        response_3 = self.client.get(response_2['Location'])
        self.assertContains(response_3,
                            "1 IPN object(s) re-verified")

        ipn_obj = PayPalIPN.objects.get()
        self.assertEqual(ipn_obj.flag, False)

        self.assertTrue(self.got_signal)



import inspect
import logging
import sys
import warnings

logger = logging.getLogger(__name__)


def warn_untested():
    msg = ("This method (or branch) is not covered by automated tests. It is therefore very "
           "vulnerable to being accidentally broken by future versions of django-paypal. "
           "Please contribute tests to ensure future functionality!")
    warnings.warn(msg, stacklevel=2)
    f = sys._getframe(1)
    source = "{0}:{1}".format(inspect.getmodule(f).__name__,
                              f.f_lineno)
    logger.warning("{0}: {1}".format(source, msg))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sys

from django import forms

if sys.version_info < (3, 0):
    from django.utils.translation import ugettext_lazy as _
else:
    from django.utils.translation import gettext_lazy as _

from paypal.pro.exceptions import PayPalFailure
from paypal.pro.fields import CountryField, CreditCardCVV2Field, CreditCardExpiryField, CreditCardField
from paypal.utils import warn_untested


class PaymentForm(forms.Form):
    """Form used to process direct payments."""
    firstname = forms.CharField(max_length=255, label=_("First Name"))
    lastname = forms.CharField(max_length=255, label=_("Last Name"))
    street = forms.CharField(max_length=255, label=_("Street Address"))
    city = forms.CharField(max_length=255, label=_("City"))
    state = forms.CharField(max_length=255, label=_("State"))
    countrycode = CountryField(label=_("Country"), initial="US")
    zip = forms.CharField(max_length=32, label=_("Postal / Zip Code"))
    acct = CreditCardField(label=_("Credit Card Number"))
    expdate = CreditCardExpiryField(label=_("Expiration Date"))
    cvv2 = CreditCardCVV2Field(label=_("Card Security Code"))
    currencycode = forms.CharField(widget=forms.HiddenInput(), initial="USD")

    def process(self, request, item):
        """Process a PayPal direct payment."""
        warn_untested()
        from paypal.pro.helpers import PayPalWPP

        wpp = PayPalWPP(request)
        params = self.cleaned_data
        params['creditcardtype'] = self.fields['acct'].card_type
        params['expdate'] = self.cleaned_data['expdate'].strftime("%m%Y")
        params['ipaddress'] = request.META.get("REMOTE_ADDR", "")
        params.update(item)

        try:
            # Create single payment:
            if 'billingperiod' not in params:
                wpp.doDirectPayment(params)
            # Create recurring payment:
            else:
                wpp.createRecurringPaymentsProfile(params, direct=True)
        except PayPalFailure:
            return False
        return True


class ConfirmForm(forms.Form):
    """Hidden form used by ExpressPay flow to keep track of payer information."""
    token = forms.CharField(max_length=255, widget=forms.HiddenInput())
    PayerID = forms.CharField(max_length=255, widget=forms.HiddenInput())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from paypal.pro.models import PayPalNVP


class PayPalNVPAdmin(admin.ModelAdmin):
    list_display = ('user', 'ipaddress', 'method', 'flag', 'flag_code', 'created_at')
    list_filter = ('flag', 'created_at')
    search_fields = ('user__email', 'ipaddress', 'flag', 'firstname', 'lastname')


admin.site.register(PayPalNVP, PayPalNVPAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-02-19 11:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pdt', '0006_auto_20160108_1112'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paypalpdt',
            name='txn_id',
            field=models.CharField(blank=True, db_index=True, help_text='PayPal transaction ID.', max_length=255, verbose_name='Transaction ID'),
        ),
        migrations.AlterField(
            model_name='paypalpdt',
            name='txn_type',
            field=models.CharField(blank=True, help_text='PayPal transaction type.', max_length=255, verbose_name='Transaction Type'),
        ),
    ]



from __future__ import unicode_literals

try:
    from django.urls import re_path
except ImportError:
    from django.conf.urls import url as re_path

from paypal.standard.pdt import views

urlpatterns = [
    re_path(r'^$', views.pdt, name="paypal-pdt"),
]



from __future__ import unicode_literals

import os
import re

from django.test import TestCase

from paypal.standard.forms import (
    PayPalEncryptedPaymentsForm, PayPalPaymentsForm, PayPalSharedSecretEncryptedPaymentsForm
)
from paypal.standard.ipn.forms import PayPalIPNForm


class PaymentsFormTest(TestCase):

    def test_form_render(self):
        return_url = 'https://example.com/return_url'

        f = PayPalPaymentsForm(initial={'business': 'me@mybusiness.com',
                                        'amount': '10.50',
                                        'shipping': '2.00',
                                        'return_url': return_url,
                                        })
        rendered = f.render()
        self.assertIn('''action="https://www.sandbox.paypal.com/cgi-bin/webscr"''', rendered)
        self.assertIn('''value="me@mybusiness.com"''', rendered)
        self.assertIn('''value="2.00"''', rendered)
        self.assertIn('''value="10.50"''', rendered)
        self.assertIn('''buynowCC''', rendered)
        self.assertIn('''value="''' + return_url + '''"''', rendered)

        f = PayPalPaymentsForm(initial={'business': 'me@mybusiness.com',
                                        'amount': '10.50',
                                        'shipping': '2.00',
                                        'return': return_url,
                                        })
        rendered = f.render()
        self.assertIn('''action="https://www.sandbox.paypal.com/cgi-bin/webscr"''', rendered)
        self.assertIn('''value="me@mybusiness.com"''', rendered)
        self.assertIn('''value="2.00"''', rendered)
        self.assertIn('''value="10.50"''', rendered)
        self.assertIn('''buynowCC''', rendered)
        self.assertIn('''value="''' + return_url + '''"''', rendered)

    def test_form_endpont(self):
        with self.settings(PAYPAL_TEST=False):
            f = PayPalPaymentsForm(initial={})
            self.assertNotIn('sandbox', f.render())

    def test_invalid_date_format(self):
        data = {'payment_date': "2015-10-25 01:21:32"}
        form = PayPalIPNForm(data)
        self.assertFalse(form.is_valid())
        self.assertIn(
            form.errors,
            [
                {
                    'payment_date': ['Invalid date format '
                                     '2015-10-25 01:21:32: '
                                     'need more than 2 values to unpack']
                },
                {
                    'payment_date': ['Invalid date format '
                                     '2015-10-25 01:21:32: '
                                     'not enough values to unpack '
                                     '(expected 5, got 2)']
                }
            ]
        )

    def test_encrypted_button(self):
        data = {
            'amount': '10.50',
            'shipping': '2.00',
        }

        # Paypal Certificate Information
        here = os.path.dirname(os.path.abspath(__file__))
        paypal_private_cert = os.path.join(here, 'test_cert__do_not_use__private.pem')
        paypal_public_cert = os.path.join(here, 'test_cert__do_not_use__public.pem')
        paypal_cert = os.path.join(here, 'test_cert_from_paypal__do_not_use.pem')
        paypal_cert_id = 'another-paypal-id'

        # Create the instance.
        form = PayPalEncryptedPaymentsForm(
            initial=data,
            private_cert=paypal_private_cert,
            public_cert=paypal_public_cert,
            paypal_cert=paypal_cert,
            cert_id=paypal_cert_id
        )
        rendered = form.render()

        self.assertIn('''name="cmd" value="_s-xclick"''', rendered)
        self.assertIn('''name="encrypted" value="-----BEGIN PKCS7-----''', rendered)
        expected_regex = re.compile(
            r'.*name="encrypted" value="-----BEGIN PKCS7-----\n' +
            r'([A-Za-z0-9/\+]{64}\n)*[A-Za-z0-9/\+]{1,64}={0,2}\n' +
            r'-----END PKCS7-----\n.*'
        )
        self.assertTrue(
            expected_regex.search(rendered),
            msg='Button encryption has wrong form - expected a block of PKCS7 data'
        )

    def test_shared_secret_encrypted_button(self):
        data = {
            'notify_url': 'https://example.com/notify_url',
            'amount': '10.50',
            'shipping': '2.00',
        }

        # Paypal Certificate Information
        here = os.path.dirname(os.path.abspath(__file__))
        paypal_private_cert = os.path.join(here, 'test_cert__do_not_use__private.pem')
        paypal_public_cert = os.path.join(here, 'test_cert__do_not_use__public.pem')
        paypal_cert = os.path.join(here, 'test_cert_from_paypal__do_not_use.pem')
        paypal_cert_id = 'another-paypal-id'

        # Create the instance.
        form = PayPalSharedSecretEncryptedPaymentsForm(
            initial=data,
            private_cert=paypal_private_cert,
            public_cert=paypal_public_cert,
            paypal_cert=paypal_cert,
            cert_id=paypal_cert_id
        )
        rendered = form.render()

        self.assertIn('''name="cmd" value="_s-xclick"''', rendered)
        self.assertIn('''name="encrypted" value="-----BEGIN PKCS7-----''', rendered)
        expected_regex = re.compile(
            r'.*name="encrypted" value="-----BEGIN PKCS7-----\n' +
            r'([A-Za-z0-9/\+]{64}\n)*[A-Za-z0-9/\+]{1,64}={0,2}\n' +
            r'-----END PKCS7-----\n.*'
        )
        self.assertTrue(
            expected_regex.search(rendered),
            msg='Button encryption has wrong form - expected a block of PKCS7 data'
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import datetime
import logging
import pprint
import time

import requests
from django.conf import settings
from django.forms.models import fields_for_model
from django.http import QueryDict
from django.utils import timezone
from django.utils.functional import cached_property
from django.utils.http import urlencode

from paypal.pro.exceptions import PayPalFailure
from paypal.pro.models import PayPalNVP
from paypal.utils import warn_untested

USER = settings.PAYPAL_WPP_USER
PASSWORD = settings.PAYPAL_WPP_PASSWORD
SIGNATURE = settings.PAYPAL_WPP_SIGNATURE
VERSION = 116.0
BASE_PARAMS = dict(USER=USER, PWD=PASSWORD, SIGNATURE=SIGNATURE, VERSION=VERSION)
ENDPOINT = "https://api-3t.paypal.com/nvp"
SANDBOX_ENDPOINT = "https://api-3t.sandbox.paypal.com/nvp"

EXPRESS_ENDPOINT = "https://www.paypal.com/webscr?cmd=_express-checkout&%s"
SANDBOX_EXPRESS_ENDPOINT = "https://www.sandbox.paypal.com/webscr?cmd=_express-checkout&%s"


log = logging.getLogger('paypal.pro')


def paypal_time(time_obj=None):
    """Returns a time suitable for PayPal time fields."""
    warn_untested()
    if time_obj is None:
        time_obj = time.gmtime()
    return time.strftime(PayPalNVP.TIMESTAMP_FORMAT, time_obj)


def paypaltime2datetime(s):
    """Convert a PayPal time string to a DateTime."""
    naive = datetime.datetime.strptime(s, PayPalNVP.TIMESTAMP_FORMAT)
    if not settings.USE_TZ:
        return naive
    else:
        # TIMESTAMP_FORMAT is UTC
        return timezone.make_aware(naive, timezone.utc)


class PayPalError(TypeError):
    """Error thrown when something is wrong."""


def express_endpoint():
    if getattr(settings, 'PAYPAL_TEST', True):
        return SANDBOX_EXPRESS_ENDPOINT
    else:
        return EXPRESS_ENDPOINT


def express_endpoint_for_token(token, commit=False):
    """
    Returns the PayPal Express Checkout endpoint for a token.
    Pass 'commit=True' if you will not prompt for confirmation when the user
    returns to your site.
    """
    pp_params = dict(token=token)
    if commit:
        pp_params['useraction'] = 'commit'
    return express_endpoint() % urlencode(pp_params)


def strip_ip_port(ip_address):
    """
    Strips the port from an IPv4 or IPv6 address, returns a unicode object.
    """

    # IPv4 with or without port
    if '.' in ip_address:
        cleaned_ip = ip_address.split(':')[0]

    # IPv6 with port
    elif ']:' in ip_address:
        # Remove the port following last ':', and then strip first and last chars for [].
        cleaned_ip = ip_address.rpartition(':')[0][1:-1]

    # IPv6 without port
    else:
        cleaned_ip = ip_address

    return cleaned_ip


class PayPalWPP(object):
    """
    Wrapper class for the PayPal Website Payments Pro.

    Website Payments Pro Integration Guide:
    https://cms.paypal.com/cms_content/US/en_US/files/developer/PP_WPP_IntegrationGuide.pdf

    Name-Value Pair API Developer Guide and Reference:
    https://cms.paypal.com/cms_content/US/en_US/files/developer/PP_NVPAPI_DeveloperGuide.pdf
    """

    def __init__(self, request=None, params=BASE_PARAMS):
        """Required - USER / PWD / SIGNATURE / VERSION"""
        self.request = request
        if getattr(settings, 'PAYPAL_TEST', True):
            self.endpoint = SANDBOX_ENDPOINT
        else:
            self.endpoint = ENDPOINT
        self.signature_values = params
        self.signature = urlencode(self.signature_values) + "&"

    @cached_property
    def NVP_FIELDS(self):
        # Put this onto class and load lazily, because in some cases there is an
        # import order problem if we put it at module level.
        return list(fields_for_model(PayPalNVP).keys())

    def doDirectPayment(self, params):
        """Call PayPal DoDirectPayment method."""
        defaults = {"method": "DoDirectPayment", "paymentaction": "Sale"}
        required = ["creditcardtype",
                    "acct",
                    "expdate",
                    "cvv2",
                    "ipaddress",
                    "firstname",
                    "lastname",
                    "street",
                    "city",
                    "state",
                    "countrycode",
                    "zip",
                    "amt",
                    ]
        nvp_obj = self._fetch(params, required, defaults)
        if nvp_obj.flag:
            raise PayPalFailure(nvp_obj.flag_info, nvp=nvp_obj)
        # @@@ Could check cvv2match / avscode are both 'X' or '0'
        # qd = django.http.QueryDict(nvp_obj.response)
        # if qd.get('cvv2match') not in ['X', '0']:
        #   nvp_obj.set_flag("Invalid cvv2match: %s" % qd.get('cvv2match')
        # if qd.get('avscode') not in ['X', '0']:
        #   nvp_obj.set_flag("Invalid avscode: %s" % qd.get('avscode')
        return nvp_obj

    def setExpressCheckout(self, params):
        """
        Initiates an Express Checkout transaction.
        Optionally, the SetExpressCheckout API operation can set up billing agreements for
        reference transactions and recurring payments.
        Returns a NVP instance - check for token and payerid to continue!
        """
        if self._is_recurring(params):
            params = self._recurring_setExpressCheckout_adapter(params)

        defaults = {"method": "SetExpressCheckout", "noshipping": 1}
        required = ["returnurl", "cancelurl", "paymentrequest_0_amt"]
        nvp_obj = self._fetch(params, required, defaults)
        if nvp_obj.flag:
            raise PayPalFailure(nvp_obj.flag_info, nvp=nvp_obj)
        return nvp_obj

    def doExpressCheckoutPayment(self, params):
        """
        Check the dude out:
        """
        defaults = {"method": "DoExpressCheckoutPayment", "paymentaction": "Sale"}
        required = ["paymentrequest_0_amt", "token", "payerid"]
        nvp_obj = self._fetch(params, required, defaults)
        if nvp_obj.flag:
            raise PayPalFailure(nvp_obj.flag_info, nvp=nvp_obj)
        return nvp_obj

    def createRecurringPaymentsProfile(self, params, direct=False):
        """
        Set direct to True to indicate that this is being called as a directPayment.
        Returns True PayPal successfully creates the profile otherwise False.
        """
        defaults = {"method": "CreateRecurringPaymentsProfile"}
        required = ["profilestartdate", "billingperiod", "billingfrequency", "amt"]

        # Direct payments require CC data
        if direct:
            required + ["creditcardtype", "acct", "expdate", "firstname", "lastname"]
        else:
            required + ["token", "payerid"]

        nvp_obj = self._fetch(params, required, defaults)

        # Flag if profile_type != ActiveProfile
        if nvp_obj.flag:
            raise PayPalFailure(nvp_obj.flag_info, nvp=nvp_obj)
        return nvp_obj

    def getExpressCheckoutDetails(self, params):
        defaults = {"method": "GetExpressCheckoutDetails"}
        required = ["token"]
        nvp_obj = self._fetch(params, required, defaults)
        if nvp_obj.flag:
            raise PayPalFailure(nvp_obj.flag_info, nvp=nvp_obj)
        return nvp_obj

    def createBillingAgreement(self, params):
        """
        Create a billing agreement for future use, without any initial payment
        """
        defaults = {"method": "CreateBillingAgreement"}
        required = ["token"]
        nvp_obj = self._fetch(params, required, defaults)
        if nvp_obj.flag:
            raise PayPalFailure(nvp_obj.flag_info, nvp=nvp_obj)
        return nvp_obj

    def getTransactionDetails(self, params):
        defaults = {"method": "GetTransactionDetails"}
        required = ["transactionid"]

        nvp_obj = self._fetch(params, required, defaults)
        if nvp_obj.flag:
            raise PayPalFailure(nvp_obj.flag_info, nvp=nvp_obj)
        return nvp_obj

    def massPay(self, params):
        raise NotImplementedError

    def getRecurringPaymentsProfileDetails(self, params):
        raise NotImplementedError

    def updateRecurringPaymentsProfile(self, params):
        defaults = {"method": "UpdateRecurringPaymentsProfile"}
        required = ["profileid"]

        nvp_obj = self._fetch(params, required, defaults)
        if nvp_obj.flag:
            raise PayPalFailure(nvp_obj.flag_info, nvp=nvp_obj)
        return nvp_obj

    def billOutstandingAmount(self, params):
        raise NotImplementedError

    def manangeRecurringPaymentsProfileStatus(self, params, fail_silently=False):
        """
        Requires `profileid` and `action` params.
        Action must be either "Cancel", "Suspend", or "Reactivate".
        """
        defaults = {"method": "ManageRecurringPaymentsProfileStatus"}
        required = ["profileid", "action"]

        nvp_obj = self._fetch(params, required, defaults)

        # TODO: This fail silently check should be using the error code, but its not easy to access
        flag_info_test_string = 'Invalid profile status for cancel action; profile should be active or suspended'
        if nvp_obj.flag and not (fail_silently and nvp_obj.flag_info == flag_info_test_string):
            raise PayPalFailure(nvp_obj.flag_info, nvp=nvp_obj)
        return nvp_obj

    def refundTransaction(self, params):
        raise NotImplementedError

    def doReferenceTransaction(self, params):
        """
        Process a payment from a buyer's account, identified by a previous
        transaction.
        The `paymentaction` param defaults to "Sale", but may also contain the
        values "Authorization" or "Order".
        """
        defaults = {"method": "DoReferenceTransaction",
                    "paymentaction": "Sale"}
        required = ["referenceid", "amt"]

        nvp_obj = self._fetch(params, required, defaults)
        if nvp_obj.flag:
            raise PayPalFailure(nvp_obj.flag_info, nvp=nvp_obj)
        return nvp_obj

    def _is_recurring(self, params):
        """Returns True if the item passed is a recurring transaction."""
        return 'billingfrequency' in params

    def _recurring_setExpressCheckout_adapter(self, params):
        """
        The recurring payment interface to SEC is different than the recurring payment
        interface to ECP. This adapts a normal call to look like a SEC call.
        """
        params['l_billingtype0'] = "RecurringPayments"
        params['l_billingagreementdescription0'] = params['desc']

        REMOVE = ["billingfrequency", "billingperiod", "profilestartdate", "desc"]
        for k in params.keys():
            if k in REMOVE:
                del params[k]

        return params

    def _fetch(self, params, required, defaults):
        """Make the NVP request and store the response."""
        defaults.update(params)
        pp_params = self._check_and_update_params(required, defaults)
        pp_string = self.signature + urlencode(pp_params)
        response = self._request(pp_string)
        response_params = self._parse_response(response)

        log.debug('PayPal Request:\n%s\n', pprint.pformat(defaults))
        log.debug('PayPal Response:\n%s\n', pprint.pformat(response_params))

        # Gather all NVP parameters to pass to a new instance.
        nvp_params = {}
        tmpd = defaults.copy()
        tmpd.update(response_params)
        for k, v in tmpd.items():
            if k in self.NVP_FIELDS:
                nvp_params[str(k)] = v

        # PayPal timestamp has to be formatted.
        if 'timestamp' in nvp_params:
            nvp_params['timestamp'] = paypaltime2datetime(nvp_params['timestamp'])

        nvp_obj = PayPalNVP(**nvp_params)
        nvp_obj.init(self.request, params, response_params)
        nvp_obj.save()
        return nvp_obj

    def _request(self, data):
        """Moved out to make testing easier."""
        return requests.post(self.endpoint, data=data.encode("ascii")).content

    def _check_and_update_params(self, required, params):
        """
        Ensure all required parameters were passed to the API call and format
        them correctly.
        """
        for r in required:
            if r not in params:
                raise PayPalError("Missing required param: %s" % r)

        # Upper case all the parameters for PayPal.
        return (dict((k.upper(), v) for k, v in params.items()))

    def _parse_response(self, response):
        """Turn the PayPal response into a dict"""
        q = QueryDict(response, encoding='UTF-8').dict()
        return {k.lower(): v for k, v in q.items()}



from django.conf import settings


class PayPalSettingsError(Exception):
    """Raised when settings be bad."""


# API Endpoints.
POSTBACK_ENDPOINT = "https://ipnpb.paypal.com/cgi-bin/webscr"
SANDBOX_POSTBACK_ENDPOINT = "https://ipnpb.sandbox.paypal.com/cgi-bin/webscr"

# Login endpoints
LOGIN_URL = "https://www.paypal.com/cgi-bin/webscr"
SANDBOX_LOGIN_URL = "https://www.sandbox.paypal.com/cgi-bin/webscr"

# Images
BUY_BUTTON_IMAGE = getattr(settings, "PAYPAL_BUY_BUTTON_IMAGE",
                           "https://www.paypal.com/en_US/i/btn/btn_buynowCC_LG.gif")
SUBSCRIPTION_BUTTON_IMAGE = getattr(settings, "PAYPAL_SUBSCRIPTION_BUTTON_IMAGE",
                             "https://www.paypal.com/en_US/i/btn/btn_subscribeCC_LG.gif")
DONATION_BUTTON_IMAGE = getattr(settings, "PAYPAL_DONATION_BUTTON_IMAGE",
                                "https://www.paypal.com/en_US/i/btn/btn_donateCC_LG.gif")


# Paypal Encrypt Certificate
PAYPAL_PRIVATE_CERT = getattr(settings, 'PAYPAL_PRIVATE_CERT', None)
PAYPAL_PUBLIC_CERT = getattr(settings, 'PAYPAL_PUBLIC_CERT', None)
PAYPAL_CERT = getattr(settings, 'PAYPAL_CERT', None)
PAYPAL_CERT_ID = getattr(settings, 'PAYPAL_CERT_ID', None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import requests
from django.conf import settings
from django.db import models
from django.http import QueryDict
from six.moves.urllib.parse import unquote_plus

from paypal.standard.conf import POSTBACK_ENDPOINT, SANDBOX_POSTBACK_ENDPOINT
from paypal.standard.models import PayPalStandardBase
from paypal.utils import warn_untested


# ### Todo: Move this logic to conf.py:
# if paypal.standard.pdt is in installed apps
# ... then check for this setting in conf.py
class PayPalSettingsError(Exception):
    """Raised when settings are incorrect."""


try:
    IDENTITY_TOKEN = settings.PAYPAL_IDENTITY_TOKEN
except AttributeError:
    raise PayPalSettingsError(
        "You must set PAYPAL_IDENTITY_TOKEN in settings.py. Get this token by enabling PDT in your PayPal account.")


class PayPalPDT(PayPalStandardBase):
    format = u"<PDT: %s %s>"

    amt = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    cm = models.CharField(max_length=255, blank=True)
    sig = models.CharField(max_length=255, blank=True)
    tx = models.CharField(max_length=255, blank=True)
    st = models.CharField(max_length=32, blank=True)

    class Meta:
        db_table = "paypal_pdt"
        verbose_name = "PayPal PDT"

    def _postback(self):
        """
        Perform PayPal PDT Postback validation.
        Sends the transaction ID and business token to PayPal which responses with
        SUCCESS or FAILED.

        """
        return requests.post(self.get_endpoint(),
                             data=dict(cmd="_notify-synch", at=IDENTITY_TOKEN, tx=self.tx)).content

    def get_endpoint(self):
        if getattr(settings, 'PAYPAL_TEST', True):
            return SANDBOX_POSTBACK_ENDPOINT
        else:
            return POSTBACK_ENDPOINT

    def _verify_postback(self):
        # ### Now we don't really care what result was, just whether a flag was set or not.
        from paypal.standard.pdt.forms import PayPalPDTForm

        response_list = self.response.split('\n')
        response_dict = {}
        for i, line in enumerate(response_list):
            unquoted_line = unquote_plus(line).strip()
            if i == 0:
                self.st = unquoted_line
            else:
                if self.st != "SUCCESS":
                    warn_untested()
                    self.set_flag(line)
                    break
                try:
                    if not unquoted_line.startswith(' -'):
                        k, v = unquoted_line.split('=')
                        response_dict[k.strip()] = v.strip()
                except ValueError:
                    pass

        qd = QueryDict('', mutable=True)
        qd.update(response_dict)
        qd.update(dict(ipaddress=self.ipaddress, st=self.st, flag_info=self.flag_info, flag=self.flag,
                       flag_code=self.flag_code))
        pdt_form = PayPalPDTForm(qd, instance=self)
        pdt_form.save(commit=False)

    def __repr__(self):
        return '<PayPalPDT id:{0}>'.format(self.id)

    def __str__(self):
        return "PayPalPDT: {0}".format(self.id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from paypal.standard.forms import PayPalStandardBaseForm
from paypal.standard.ipn.models import PayPalIPN


class PayPalIPNForm(PayPalStandardBaseForm):
    """
    Form used to receive and record PayPal IPN notifications.

    PayPal IPN test tool:
    https://developer.paypal.com/us/cgi-bin/devscr?cmd=_tools-session
    """

    class Meta:
        model = PayPalIPN
        exclude = []



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.functional import cached_property

from paypal.standard.conf import POSTBACK_ENDPOINT, SANDBOX_POSTBACK_ENDPOINT
from paypal.standard.helpers import check_secret, duplicate_txn_id
from paypal.utils import warn_untested

ST_PP_ACTIVE = 'Active'
ST_PP_CANCELLED = 'Cancelled'
ST_PP_CANCELED_REVERSAL = 'Canceled_Reversal'
ST_PP_CLEARED = 'Cleared'
ST_PP_COMPLETED = 'Completed'
ST_PP_CREATED = 'Created'
ST_PP_DECLINED = 'Declined'
ST_PP_DENIED = 'Denied'
ST_PP_EXPIRED = 'Expired'
ST_PP_FAILED = 'Failed'
ST_PP_PAID = 'Paid'
ST_PP_PENDING = 'Pending'
ST_PP_PROCESSED = 'Processed'
ST_PP_REFUNDED = 'Refunded'
ST_PP_REFUSED = 'Refused'
ST_PP_REVERSED = 'Reversed'
ST_PP_REWARDED = 'Rewarded'
ST_PP_UNCLAIMED = 'Unclaimed'
ST_PP_UNCLEARED = 'Uncleared'
ST_PP_VOIDED = 'Voided'

try:
    from idmapper.models import SharedMemoryModel as Model
except ImportError:
    Model = models.Model


DEFAULT_ENCODING = 'windows-1252'  # PayPal seems to normally use this.


class PayPalStandardBase(Model):
    """Base class for common variables shared by IPN and PDT"""
    # See https://developer.paypal.com/docs/classic/ipn/integration-guide/IPNandPDTVariables/

    # @@@ Might want to add all these one distant day.
    # FLAG_CODE_CHOICES = (
    # PAYMENT_STATUS_CHOICES = "Canceled_ Reversal Completed Denied Expired " \
    #                          "Failed Pending Processed Refunded Reversed Voided".split()
    PAYMENT_STATUS_CHOICES = [ST_PP_ACTIVE,
                              ST_PP_CANCELLED,
                              ST_PP_CANCELED_REVERSAL,
                              ST_PP_CLEARED,
                              ST_PP_COMPLETED,
                              ST_PP_CREATED,
                              ST_PP_DECLINED,
                              ST_PP_DENIED,
                              ST_PP_EXPIRED,
                              ST_PP_FAILED,
                              ST_PP_PAID,
                              ST_PP_PENDING,
                              ST_PP_PROCESSED,
                              ST_PP_REFUNDED,
                              ST_PP_REFUSED,
                              ST_PP_REVERSED,
                              ST_PP_REWARDED,
                              ST_PP_UNCLAIMED,
                              ST_PP_UNCLEARED,
                              ST_PP_VOIDED,
                              ]
    # AUTH_STATUS_CHOICES = "Completed Pending Voided".split()
    # ADDRESS_STATUS_CHOICES = "confirmed unconfirmed".split()
    # PAYER_STATUS_CHOICES = "verified / unverified".split()
    # PAYMENT_TYPE_CHOICES =  "echeck / instant.split()
    # PENDING_REASON = "address authorization echeck intl multi-currency unilateral upgrade verify other".split()
    # REASON_CODE = "chargeback guarantee buyer_complaint refund other".split()
    # TRANSACTION_ENTITY_CHOICES = "auth reauth order payment".split()

    # Transaction and Notification-Related Variables
    business = models.CharField(max_length=127, blank=True, help_text="Email where the money was sent.")
    charset = models.CharField(max_length=255, blank=True)
    custom = models.CharField(max_length=256, blank=True)
    notify_version = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    parent_txn_id = models.CharField("Parent Transaction ID", max_length=19, blank=True)
    receiver_email = models.EmailField(max_length=254, blank=True)
    receiver_id = models.CharField(max_length=255, blank=True)  # 258DLEHY2BDK6
    residence_country = models.CharField(max_length=2, blank=True)
    test_ipn = models.BooleanField(default=False, blank=True)
    txn_id = models.CharField("Transaction ID", max_length=255, blank=True, help_text="PayPal transaction ID.",
                              db_index=True)
    txn_type = models.CharField("Transaction Type", max_length=255, blank=True, help_text="PayPal transaction type.")
    verify_sign = models.CharField(max_length=255, blank=True)

    # Buyer Information Variables
    address_country = models.CharField(max_length=64, blank=True)
    address_city = models.CharField(max_length=40, blank=True)
    address_country_code = models.CharField(max_length=64, blank=True, help_text="ISO 3166")
    address_name = models.CharField(max_length=128, blank=True)
    address_state = models.CharField(max_length=40, blank=True)
    address_status = models.CharField(max_length=255, blank=True)
    address_street = models.CharField(max_length=200, blank=True)
    address_zip = models.CharField(max_length=20, blank=True)
    contact_phone = models.CharField(max_length=20, blank=True)
    first_name = models.CharField(max_length=64, blank=True)
    last_name = models.CharField(max_length=64, blank=True)
    payer_business_name = models.CharField(max_length=127, blank=True)
    payer_email = models.CharField(max_length=127, blank=True)
    payer_id = models.CharField(max_length=13, blank=True)

    # Payment Information Variables
    auth_amount = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    auth_exp = models.CharField(max_length=28, blank=True)
    auth_id = models.CharField(max_length=19, blank=True)
    auth_status = models.CharField(max_length=255, blank=True)
    exchange_rate = models.DecimalField(max_digits=64, decimal_places=16, default=0, blank=True, null=True)
    invoice = models.CharField(max_length=127, blank=True)
    item_name = models.CharField(max_length=127, blank=True)
    item_number = models.CharField(max_length=127, blank=True)
    mc_currency = models.CharField(max_length=32, default="USD", blank=True)
    mc_fee = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    mc_gross = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    mc_handling = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    mc_shipping = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    memo = models.CharField(max_length=255, blank=True)
    num_cart_items = models.IntegerField(blank=True, default=0, null=True)
    option_name1 = models.CharField(max_length=64, blank=True)
    option_name2 = models.CharField(max_length=64, blank=True)
    option_selection1 = models.CharField(max_length=200, blank=True)
    option_selection2 = models.CharField(max_length=200, blank=True)
    payer_status = models.CharField(max_length=255, blank=True)
    payment_date = models.DateTimeField(blank=True, null=True, help_text="HH:MM:SS DD Mmm YY, YYYY PST")
    payment_gross = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    payment_status = models.CharField(max_length=255, blank=True)
    payment_type = models.CharField(max_length=255, blank=True)
    pending_reason = models.CharField(max_length=255, blank=True)
    protection_eligibility = models.CharField(max_length=255, blank=True)
    quantity = models.IntegerField(blank=True, default=1, null=True)
    reason_code = models.CharField(max_length=255, blank=True)
    remaining_settle = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    settle_amount = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    settle_currency = models.CharField(max_length=32, blank=True)
    shipping = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    shipping_method = models.CharField(max_length=255, blank=True)
    tax = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    transaction_entity = models.CharField(max_length=255, blank=True)

    # Auction Variables
    auction_buyer_id = models.CharField(max_length=64, blank=True)
    auction_closing_date = models.DateTimeField(blank=True, null=True, help_text="HH:MM:SS DD Mmm YY, YYYY PST")
    auction_multi_item = models.IntegerField(blank=True, default=0, null=True)
    for_auction = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)

    # Recurring Payments Variables
    amount = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    amount_per_cycle = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    initial_payment_amount = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    next_payment_date = models.DateTimeField(blank=True, null=True, help_text="HH:MM:SS DD Mmm YY, YYYY PST")
    outstanding_balance = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    payment_cycle = models.CharField(max_length=255, blank=True)  # Monthly
    period_type = models.CharField(max_length=255, blank=True)
    product_name = models.CharField(max_length=255, blank=True)
    product_type = models.CharField(max_length=255, blank=True)
    profile_status = models.CharField(max_length=255, blank=True)
    recurring_payment_id = models.CharField(max_length=255, blank=True)  # I-FA4XVST722B9
    rp_invoice_id = models.CharField(max_length=127, blank=True)  # 1335-7816-2936-1451
    time_created = models.DateTimeField(blank=True, null=True, help_text="HH:MM:SS DD Mmm YY, YYYY PST")

    # Subscription Variables
    amount1 = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    amount2 = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    amount3 = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    mc_amount1 = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    mc_amount2 = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    mc_amount3 = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    password = models.CharField(max_length=24, blank=True)
    period1 = models.CharField(max_length=255, blank=True)
    period2 = models.CharField(max_length=255, blank=True)
    period3 = models.CharField(max_length=255, blank=True)
    reattempt = models.CharField(max_length=1, blank=True)
    recur_times = models.IntegerField(blank=True, default=0, null=True)
    recurring = models.CharField(max_length=1, blank=True)
    retry_at = models.DateTimeField(blank=True, null=True, help_text="HH:MM:SS DD Mmm YY, YYYY PST")
    subscr_date = models.DateTimeField(blank=True, null=True, help_text="HH:MM:SS DD Mmm YY, YYYY PST")
    subscr_effective = models.DateTimeField(blank=True, null=True, help_text="HH:MM:SS DD Mmm YY, YYYY PST")
    subscr_id = models.CharField(max_length=19, blank=True)
    username = models.CharField(max_length=64, blank=True)

    # Billing Agreement Variables
    mp_id = models.CharField(max_length=128, blank=True, null=True)  # B-0G433009BJ555711U

    # Dispute Resolution Variables
    case_creation_date = models.DateTimeField(blank=True, null=True, help_text="HH:MM:SS DD Mmm YY, YYYY PST")
    case_id = models.CharField(max_length=255, blank=True)
    case_type = models.CharField(max_length=255, blank=True)

    # Variables not categorized
    receipt_id = models.CharField(max_length=255, blank=True)  # 1335-7816-2936-1451
    currency_code = models.CharField(max_length=32, default="USD", blank=True)
    handling_amount = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)

    # This undocumented variable apparently contains the same as the 'custom'
    # field - http://stackoverflow.com/questions/8464442/set-transaction-subject-paypal-ipn
    transaction_subject = models.CharField(max_length=256, blank=True)

    # @@@ Mass Pay Variables (Not Implemented, needs a separate model, for each transaction x)
    # fraud_managment_pending_filters_x = models.CharField(max_length=255, blank=True)
    # option_selection1_x = models.CharField(max_length=200, blank=True)
    # option_selection2_x = models.CharField(max_length=200, blank=True)
    # masspay_txn_id_x = models.CharField(max_length=19, blank=True)
    # mc_currency_x = models.CharField(max_length=32, default="USD", blank=True)
    # mc_fee_x = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    # mc_gross_x = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    # mc_handlingx = models.DecimalField(max_digits=64, decimal_places=2, default=0, blank=True, null=True)
    # payment_date = models.DateTimeField(blank=True, null=True, help_text="HH:MM:SS DD Mmm YY, YYYY PST")
    # payment_status = models.CharField(max_length=9, blank=True)
    # reason_code = models.CharField(max_length=15, blank=True)
    # receiver_email_x = models.EmailField(max_length=127, blank=True)
    # status_x = models.CharField(max_length=9, blank=True)
    # unique_id_x = models.CharField(max_length=13, blank=True)

    # Non-PayPal Variables - full IPN/PDT query and time fields.
    ipaddress = models.GenericIPAddressField(blank=True, null=True)
    flag = models.BooleanField(default=False, blank=True)
    flag_code = models.CharField(max_length=16, blank=True)
    flag_info = models.TextField(blank=True)
    query = models.TextField(blank=True)  # What Paypal sent to us initially
    response = models.TextField(blank=True)  # What we got back from our request
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    # Where did it come from?
    from_view = models.CharField(max_length=6, null=True, blank=True)

    class Meta:
        abstract = True
        app_label = 'paypal_standard_base'

    def __unicode__(self):
        if self.is_transaction():
            return self.format % ("Transaction", self.txn_id)
        elif self.is_subscription():
            return self.format % ("Subscription", self.subscr_id)
        else:
            return self.format % ("Recurring", self.recurring_payment_id)

    @cached_property
    def posted_data_dict(self):
        """
        All the data that PayPal posted to us, as a correctly parsed dictionary of values.
        """
        if not self.query:
            return None
        from django.http import QueryDict
        roughdecode = dict(item.split('=', 1) for item in self.query.split('&'))
        encoding = roughdecode.get('charset', None)
        if encoding is None:
            encoding = DEFAULT_ENCODING
        query = self.query.encode('ascii')
        data = QueryDict(query, encoding=encoding)
        return data.dict()

    def is_transaction(self):
        return len(self.txn_id) > 0

    def is_refund(self):
        return self.payment_status == ST_PP_REFUNDED

    def is_reversed(self):
        return self.payment_status == ST_PP_REVERSED

    def is_recurring(self):
        return len(self.recurring_payment_id) > 0

    def is_subscription(self):
        warn_untested()
        return len(self.subscr_id) > 0

    def is_subscription_payment(self):
        warn_untested()
        return self.txn_type == "subscr_payment"

    def is_subscription_failed(self):
        warn_untested()
        return self.txn_type == "subscr_failed"

    def is_subscription_cancellation(self):
        warn_untested()
        return self.txn_type == "subscr_cancel"

    def is_subscription_end_of_term(self):
        warn_untested()
        return self.txn_type == "subscr_eot"

    def is_subscription_modified(self):
        warn_untested()
        return self.txn_type == "subscr_modify"

    def is_subscription_signup(self):
        warn_untested()
        return self.txn_type == "subscr_signup"

    def is_recurring_create(self):
        return self.txn_type == "recurring_payment_profile_created"

    def is_recurring_payment(self):
        return self.txn_type == "recurring_payment"

    def is_recurring_cancel(self):
        return self.txn_type == "recurring_payment_profile_cancel"

    def is_recurring_skipped(self):
        return self.txn_type == "recurring_payment_skipped"

    def is_recurring_failed(self):
        return self.txn_type == "recurring_payment_failed"

    def is_recurring_suspended(self):
        warn_untested()
        return self.txn_type == "recurring_payment_suspended"

    def is_recurring_suspended_due_to_max_failed_payment(self):
        warn_untested()
        return self.txn_type == "recurring_payment_suspended_due_to_max_failed_payment"

    def is_billing_agreement(self):
        warn_untested()
        return len(self.mp_id) > 0

    def is_billing_agreement_create(self):
        warn_untested()
        return self.txn_type == "mp_signup"

    def is_billing_agreement_cancel(self):
        warn_untested()
        return self.txn_type == "mp_cancel"

    def set_flag(self, info, code=None):
        """Sets a flag on the transaction and also sets a reason."""
        self.flag = True
        self.flag_info += info
        if code is not None:
            warn_untested()
            self.flag_code = code

    def clear_flag(self):
        self.flag = False
        self.flag_info = ""
        self.flag_code = ""

    def verify(self):
        """
        Verifies an IPN and a PDT.
        Checks for obvious signs of weirdness in the payment and flags appropriately.
        """
        self.response = self._postback().decode('ascii')
        self.clear_flag()
        self._verify_postback()
        if not self.flag:
            if self.is_transaction():
                if self.payment_status not in self.PAYMENT_STATUS_CHOICES:
                    self.set_flag("Invalid payment_status. (%s)" % self.payment_status)
                if duplicate_txn_id(self):
                    self.set_flag("Duplicate txn_id. (%s)" % self.txn_id)

        self.save()

    def verify_secret(self, form_instance, secret):
        """Verifies an IPN payment over SSL using EWP."""
        warn_untested()
        if not check_secret(form_instance, secret):
            self.set_flag("Invalid secret. (%s)") % secret
        self.save()

    def get_endpoint(self):
        """Set Sandbox endpoint if the test variable is present."""
        if self.test_ipn:
            return SANDBOX_POSTBACK_ENDPOINT
        else:
            return POSTBACK_ENDPOINT

    def send_signals(self):
        """Shout for the world to hear whether a txn was successful."""
        raise NotImplementedError

    def initialize(self, request):
        """Store the data we'll need to make the postback from the request object."""
        if request.method == 'GET':
            # PDT only - this data is currently unused
            self.query = request.META.get('QUERY_STRING', '')
        elif request.method == 'POST':
            # The following works if paypal sends an ASCII bytestring, which it does.
            self.query = request.body.decode('ascii')
        self.ipaddress = request.META.get('REMOTE_ADDR', '')

    def _postback(self):
        """Perform postback to PayPal and store the response in self.response."""
        raise NotImplementedError

    def _verify_postback(self):
        """Check self.response is valid andcall self.set_flag if there is an error."""
        raise NotImplementedError



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import sys
from datetime import datetime
from warnings import warn

from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils import timezone
from django.utils.html import format_html
from django.utils.safestring import mark_safe

if sys.version_info < (3, 0):
    from django.utils.translation import ugettext_lazy as _
else:
    from django.utils.translation import gettext_lazy as _

from paypal.standard.conf import (
    BUY_BUTTON_IMAGE, DONATION_BUTTON_IMAGE, LOGIN_URL, PAYPAL_CERT, PAYPAL_CERT_ID, PAYPAL_PRIVATE_CERT,
    PAYPAL_PUBLIC_CERT, SANDBOX_LOGIN_URL, SUBSCRIPTION_BUTTON_IMAGE
)
from paypal.standard.widgets import ValueHiddenInput
from paypal.utils import warn_untested

log = logging.getLogger(__name__)


# PayPal date format e.g.:
#   20:18:05 Jan 30, 2009 PST
#
# PayPal dates have been spotted in the wild with these formats, beware!
#
# %H:%M:%S %b. %d, %Y PST
# %H:%M:%S %b. %d, %Y PDT
# %H:%M:%S %b %d, %Y PST
# %H:%M:%S %b %d, %Y PDT
#
# To avoid problems with different locales, we don't rely on datetime.strptime,
# which is locale dependent, but do custom parsing in PayPalDateTimeField

MONTHS = [
    'Jan', 'Feb', 'Mar', 'Apr',
    'May', 'Jun', 'Jul', 'Aug',
    'Sep', 'Oct', 'Nov', 'Dec',
]


class PayPalDateTimeField(forms.DateTimeField):

    def to_python(self, value):
        if value in self.empty_values:
            return None

        if isinstance(value, datetime):
            return value

        value = value.strip()

        try:
            time_part, month_part, day_part, year_part, zone_part = value.split()
            month_part = month_part.strip(".")
            day_part = day_part.strip(",")
            month = MONTHS.index(month_part) + 1
            day = int(day_part)
            year = int(year_part)
            hour, minute, second = map(int, time_part.split(":"))
            dt = datetime(year, month, day, hour, minute, second)
        except ValueError as e:
            raise ValidationError(
                _("Invalid date format %(value)s: %(e)s"),
                params={'value': value, 'e': e},
                code="invalid_date"
            )

        if zone_part in ["PDT", "PST"]:
            # PST/PDT is 'US/Pacific'
            dt = timezone.pytz.timezone('US/Pacific').localize(
                dt, is_dst=zone_part == 'PDT')
            if not settings.USE_TZ:
                dt = timezone.make_naive(dt, timezone=timezone.utc)
        return dt


class PayPalPaymentsForm(forms.Form):
    """
    Creates a PayPal Payments Standard "Buy It Now" button, configured for a
    selling a single item with no shipping.

    For a full overview of all the fields you can set (there is a lot!) see:
    http://tinyurl.com/pps-integration

    Usage:
    >>> f = PayPalPaymentsForm(initial={'item_name':'Widget 001', ...})
    >>> f.render()
    u'<form action="https://www.paypal.com/cgi-bin/webscr" method="post"> ...'

    """
    CMD_CHOICES = (
        ("_xclick", "Buy now or Donations"),
        ("_donations", "Donations"),
        ("_cart", "Shopping cart"),
        ("_xclick-subscriptions", "Subscribe"),
        ("_xclick-auto-billing", "Automatic Billing"),
        ("_xclick-payment-plan", "Installment Plan"),
    )
    SHIPPING_CHOICES = ((1, "No shipping"), (0, "Shipping"))
    NO_NOTE_CHOICES = ((1, "No Note"), (0, "Include Note"))
    RECURRING_PAYMENT_CHOICES = (
        (1, "Subscription Payments Recur"),
        (0, "Subscription payments do not recur")
    )
    REATTEMPT_ON_FAIL_CHOICES = (
        (1, "reattempt billing on Failure"),
        (0, "Do Not reattempt on failure")
    )

    BUY = 'buy'
    SUBSCRIBE = 'subscribe'
    DONATE = 'donate'

    # Default fields.
    cmd = forms.ChoiceField(widget=forms.HiddenInput(), initial=CMD_CHOICES[0][0])
    charset = forms.CharField(widget=forms.HiddenInput(), initial="utf-8")
    currency_code = forms.CharField(widget=forms.HiddenInput(), initial="USD")
    no_shipping = forms.ChoiceField(widget=forms.HiddenInput(), choices=SHIPPING_CHOICES,
                                    initial=SHIPPING_CHOICES[0][0])

    def __init__(self, button_type="buy", *args, **kwargs):
        super(PayPalPaymentsForm, self).__init__(*args, **kwargs)
        self.button_type = button_type
        if 'initial' in kwargs:
            kwargs['initial'] = self._fix_deprecated_return_url(kwargs['initial'])
            # Dynamically create, so we can support everything PayPal does.
            for k, v in kwargs['initial'].items():
                if k not in self.base_fields:
                    self.fields[k] = forms.CharField(label=k, widget=ValueHiddenInput(), initial=v)

    def _fix_deprecated_return_url(self, initial_args):
        if 'return_url' in initial_args:
            warn("""The use of the initial['return_url'] is Deprecated.
                    Please use initial['return'] instead""", DeprecationWarning)
            initial_args['return'] = initial_args['return_url']
            del initial_args['return_url']
        return initial_args

    def test_mode(self):
        return getattr(settings, 'PAYPAL_TEST', True)

    def get_login_url(self):
        "Returns the endpoint url for the form."
        if self.test_mode():
            return SANDBOX_LOGIN_URL
        else:
            return LOGIN_URL

    def render(self):
        return format_html(u"""<form action="{0}" method="post">
    {1}
    <input type="image" src="{2}" name="submit" alt="Buy it Now" />
</form>""", self.get_login_url(), self.as_p(), self.get_image())

    def get_image(self):
        return {
            self.SUBSCRIBE: SUBSCRIPTION_BUTTON_IMAGE,
            self.BUY: BUY_BUTTON_IMAGE,
            self.DONATE: DONATION_BUTTON_IMAGE,
        }[self.button_type]

    def is_transaction(self):
        warn_untested()
        return not self.is_subscription()

    def is_donation(self):
        warn_untested()
        return self.button_type == self.DONATE

    def is_subscription(self):
        warn_untested()
        return self.button_type == self.SUBSCRIBE


class PayPalEncryptedPaymentsForm(PayPalPaymentsForm):
    """
    Creates a PayPal Encrypted Payments "Buy It Now" button.
    Requires the M2Crypto package.

    Based on example at:
    http://blog.mauveweb.co.uk/2007/10/10/paypal-with-django/

    """

    def __init__(self, private_cert=PAYPAL_PRIVATE_CERT, public_cert=PAYPAL_PUBLIC_CERT,
            paypal_cert=PAYPAL_CERT, cert_id=PAYPAL_CERT_ID, *args, **kwargs):
        super(PayPalEncryptedPaymentsForm, self).__init__(*args, **kwargs)
        self.private_cert = private_cert
        self.public_cert = public_cert
        self.paypal_cert = paypal_cert
        self.cert_id = cert_id

    def _encrypt(self):
        """Use your key thing to encrypt things."""
        from M2Crypto import BIO, SMIME, X509

        # Iterate through the fields and pull out the ones that have a value.
        plaintext = 'cert_id=%s\n' % self.cert_id
        for name, field in self.fields.items():
            value = None
            if name in self.initial:
                value = self.initial[name]
            elif field.initial is not None:
                value = field.initial
            if value is not None:
                plaintext += u'%s=%s\n' % (name, value)
        plaintext = plaintext.encode('utf-8')

        # Begin crypto weirdness.
        s = SMIME.SMIME()
        s.load_key_bio(BIO.openfile(self.private_cert), BIO.openfile(self.public_cert))
        p7 = s.sign(BIO.MemoryBuffer(plaintext), flags=SMIME.PKCS7_BINARY)
        x509 = X509.load_cert_bio(BIO.openfile(self.paypal_cert))
        sk = X509.X509_Stack()
        sk.push(x509)
        s.set_x509_stack(sk)
        s.set_cipher(SMIME.Cipher('des_ede3_cbc'))
        tmp = BIO.MemoryBuffer()
        p7.write_der(tmp)
        p7 = s.encrypt(tmp, flags=SMIME.PKCS7_BINARY)
        out = BIO.MemoryBuffer()
        p7.write(out)
        return out.read().decode()

    def as_p(self):
        return mark_safe(u"""
<input type="hidden" name="cmd" value="_s-xclick" />
<input type="hidden" name="encrypted" value="%s" />
        """ % self._encrypt())


class PayPalSharedSecretEncryptedPaymentsForm(PayPalEncryptedPaymentsForm):
    """
    Creates a PayPal Encrypted Payments "Buy It Now" button with a Shared Secret.
    Shared secrets should only be used when your IPN endpoint is on HTTPS.

    Adds a secret to the notify_url based on the contents of the form.

    """

    def __init__(self, *args, **kwargs):
        "Make the secret from the form initial data and slip it into the form."
        from paypal.standard.helpers import make_secret

        super(PayPalSharedSecretEncryptedPaymentsForm, self).__init__(*args, **kwargs)
        # @@@ Attach the secret parameter in a way that is safe for other query params.
        secret_param = "?secret=%s" % make_secret(self)
        # Initial data used in form construction overrides defaults
        if 'notify_url' in self.initial:
            self.initial['notify_url'] += secret_param
        else:
            self.fields['notify_url'].initial += secret_param


class PayPalStandardBaseForm(forms.ModelForm):
    """Form used to receive and record PayPal IPN/PDT."""
    # PayPal dates have non-standard formats.
    time_created = PayPalDateTimeField(required=False)
    payment_date = PayPalDateTimeField(required=False)
    next_payment_date = PayPalDateTimeField(required=False)
    subscr_date = PayPalDateTimeField(required=False)
    subscr_effective = PayPalDateTimeField(required=False)
    retry_at = PayPalDateTimeField(required=False)
    case_creation_date = PayPalDateTimeField(required=False)
    auction_closing_date = PayPalDateTimeField(required=False)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ipn', '0003_auto_20141117_1647'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paypalipn',
            name='address_status',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='auth_status',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='case_id',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='case_type',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='charset',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='payer_status',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='payment_cycle',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='payment_status',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='payment_type',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='pending_reason',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='period1',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='period2',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='period3',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='period_type',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='product_name',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='product_type',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='profile_status',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='protection_eligibility',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='reason_code',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='receipt_id',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='receiver_email',
            field=models.EmailField(max_length=254, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='receiver_id',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='recurring_payment_id',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='transaction_entity',
            field=models.CharField(max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='txn_id',
            field=models.CharField(help_text=b'PayPal transaction ID.', max_length=255, verbose_name=b'Transaction ID', db_index=True, blank=True),
        ),
        migrations.AlterField(
            model_name='paypalipn',
            name='txn_type',
            field=models.CharField(help_text=b'PayPal transaction type.', max_length=255, verbose_name=b'Transaction Type', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pdt', '0005_auto_20151217_0948'),
    ]

    operations = [
        migrations.AddField(
            model_name='paypalpdt',
            name='option_selection1',
            field=models.CharField(max_length=200, blank=True),
        ),
        migrations.AddField(
            model_name='paypalpdt',
            name='option_selection2',
            field=models.CharField(max_length=200, blank=True),
        ),
    ]



"""
Note that sometimes you will get duplicate signals emitted, depending on configuration of your systems.
If you do encounter this, you will need to add the "dispatch_uid" to your connect handlers:
http://code.djangoproject.com/wiki/Signals#Helppost_saveseemstobeemittedtwiceforeachsave

"""
from __future__ import unicode_literals

from django.dispatch import Signal

# Sent when a validated, non-duplicated IPN is received.
valid_ipn_received = Signal()

# Sent when a flagged IPN (e.g. duplicate, invalid) is received.
invalid_ipn_received = Signal()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse

from paypal.pro.exceptions import PayPalFailure
from paypal.pro.forms import ConfirmForm, PaymentForm
from paypal.pro.helpers import PayPalWPP, express_endpoint_for_token
from paypal.utils import warn_untested


class PayPalPro(object):
    """
    This class-based view takes care of PayPal WebsitePaymentsPro (WPP).
    PayPalPro has two separate flows - DirectPayment and ExpressPayFlow. In
    DirectPayment the user buys on your site. In ExpressPayFlow the user is
    direct to PayPal to confirm their purchase. PayPalPro implements both
    flows. To it create an instance using the these parameters:

    item: a dictionary that holds information about the item being purchased.

    For single item purchase (pay once):

        Required Keys:
            * amt: Float amount of the item.

        Optional Keys:
            * custom: You can set this to help you identify a transaction.
            * invnum: Unique ID that identifies this transaction.

    For recurring billing:

        Required Keys:
          * amt: Float amount for each billing cycle.
          * billingperiod: String unit of measure for the billing cycle (Day|Week|SemiMonth|Month|Year)
          * billingfrequency: Integer number of periods that make up a cycle.
          * profilestartdate: The date to begin billing. "2008-08-05T17:00:00Z" UTC/GMT
          * desc: Description of what you're billing for.

        Optional Keys:
          * trialbillingperiod: String unit of measure for trial cycle (Day|Week|SemiMonth|Month|Year)
          * trialbillingfrequency: Integer # of periods in a cycle.
          * trialamt: Float amount to bill for the trial period.
          * trialtotalbillingcycles: Integer # of cycles for the trial payment period.
          * failedinitamtaction: set to continue on failure (ContinueOnFailure / CancelOnFailure)
          * maxfailedpayments: number of payments before profile is suspended.
          * autobilloutamt: automatically bill outstanding amount.
          * subscribername: Full name of the person who paid.
          * profilereference: Unique reference or invoice number.
          * taxamt: How much tax.
          * initamt: Initial non-recurring payment due upon creation.
          * currencycode: defaults to USD
          * + a bunch of shipping fields

    payment_form_cls: form class that will be used to display the payment form.
    It should inherit from `paypal.pro.forms.PaymentForm` if you're adding more.

    payment_template: template used to ask the dude for monies. To comply with
    PayPal standards it must include a link to PayPal Express Checkout.

    confirm_form_cls: form class that will be used to display the confirmation form.
    It should inherit from `paypal.pro.forms.ConfirmForm`. It is only used in the Express flow.

    success_url / fail_url: URLs to be redirected to when the payment successful or fails.
    """
    errors = {
        "processing": "There was an error processing your payment. Check your information and try again.",
        "form": "Please correct the errors below and try again.",
        "paypal": "There was a problem contacting PayPal. Please try again later."
    }

    def __init__(self, item=None, payment_form_cls=PaymentForm,
                 payment_template="pro/payment.html", confirm_form_cls=ConfirmForm,
                 confirm_template="pro/confirm.html", success_url="?success",
                 fail_url=None, context=None, form_context_name="form", nvp_handler=None):
        self.item = item
        self.payment_form_cls = payment_form_cls
        self.payment_template = payment_template
        self.confirm_form_cls = confirm_form_cls
        self.confirm_template = confirm_template
        self.success_url = success_url
        self.fail_url = fail_url
        self.context = context or {}
        self.form_context_name = form_context_name
        self.nvp_handler = nvp_handler

    def __call__(self, request):
        """Return the appropriate response for the state of the transaction."""
        self.request = request
        if request.method == "GET":
            if self.should_redirect_to_express():
                return self.redirect_to_express()
            elif self.should_render_confirm_form():
                return self.render_confirm_form()
            elif self.should_render_payment_form():
                return self.render_payment_form()
        else:
            if self.should_validate_confirm_form():
                return self.validate_confirm_form()
            elif self.should_validate_payment_form():
                return self.validate_payment_form()

        # Default to the rendering the payment form.
        return self.render_payment_form()

    def is_recurring(self):
        return self.item is not None and 'billingperiod' in self.item

    def should_redirect_to_express(self):
        return 'express' in self.request.GET

    def should_render_confirm_form(self):
        return 'token' in self.request.GET and 'PayerID' in self.request.GET

    def should_render_payment_form(self):
        return True

    def should_validate_confirm_form(self):
        return 'token' in self.request.POST and 'PayerID' in self.request.POST

    def should_validate_payment_form(self):
        return True

    def render_payment_form(self):
        """Display the DirectPayment for entering payment information."""
        self.context[self.form_context_name] = self.payment_form_cls()
        return TemplateResponse(self.request, self.payment_template, self.context)

    def validate_payment_form(self):
        """Try to validate and then process the DirectPayment form."""
        warn_untested()
        form = self.payment_form_cls(self.request.POST)
        if form.is_valid():
            success = form.process(self.request, self.item)
            if success:
                return HttpResponseRedirect(self.success_url)
            else:
                self.context['errors'] = self.errors['processing']

        self.context[self.form_context_name] = form
        self.context.setdefault("errors", self.errors['form'])
        return TemplateResponse(self.request, self.payment_template, self.context)

    def redirect_to_express(self):
        """
        First step of ExpressCheckout. Redirect the request to PayPal using the
        data returned from setExpressCheckout.
        """
        wpp = PayPalWPP(self.request)
        try:
            nvp_obj = wpp.setExpressCheckout(self.item)
        except PayPalFailure:
            warn_untested()
            self.context['errors'] = self.errors['paypal']
            return self.render_payment_form()
        else:
            return HttpResponseRedirect(express_endpoint_for_token(nvp_obj.token))

    def render_confirm_form(self):
        """
        Second step of ExpressCheckout. Display an order confirmation form which
        contains hidden fields with the token / PayerID from PayPal.
        """
        warn_untested()
        initial = dict(token=self.request.GET['token'], PayerID=self.request.GET['PayerID'])
        self.context[self.form_context_name] = self.confirm_form_cls(initial=initial)
        return TemplateResponse(self.request, self.confirm_template, self.context)

    def validate_confirm_form(self):
        """
        Third and final step of ExpressCheckout. Request has pressed the confirmation but
        and we can send the final confirmation to PayPal using the data from the POST'ed form.
        """
        wpp = PayPalWPP(self.request)
        pp_data = dict(token=self.request.POST['token'], payerid=self.request.POST['PayerID'])
        self.item.update(pp_data)

        # @@@ This check and call could be moved into PayPalWPP.
        try:
            if self.is_recurring():
                warn_untested()
                nvp = wpp.createRecurringPaymentsProfile(self.item)
            else:
                nvp = wpp.doExpressCheckoutPayment(self.item)
            self.handle_nvp(nvp)
        except PayPalFailure:
            self.context['errors'] = self.errors['processing']
            return self.render_payment_form()
        else:
            return HttpResponseRedirect(self.success_url)

    def handle_nvp(self, nvp):
        self.nvp_handler(nvp)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from paypal.standard.pdt.models import PayPalPDT


# ToDo: How similar is this to PayPalIPNAdmin? Could we just inherit off one common admin model?
class PayPalPDTAdmin(admin.ModelAdmin):
    date_hierarchy = 'payment_date'
    fieldsets = (
        (None, {
            "fields":
                ['flag',
                 'txn_id',
                 'txn_type',
                 'payment_status',
                 'payment_date',
                 'transaction_entity',
                 'reason_code',
                 'pending_reason',
                 'mc_gross',
                 'mc_fee',
                 'auth_status',
                 'auth_amount',
                 'auth_exp',
                 'auth_id',
                 ],
        }),
        ("Address", {
            "description": "The address of the Buyer.",
            'classes': ('collapse',),
            "fields":
                ['address_city',
                 'address_country',
                 'address_country_code',
                 'address_name',
                 'address_state',
                 'address_status',
                 'address_street',
                 'address_zip',
                 ],
        }),
        ("Buyer", {
            "description": "The information about the Buyer.",
            'classes': ('collapse',),
            "fields":
                ['first_name',
                 'last_name',
                 'payer_business_name',
                 'payer_email',
                 'payer_id',
                 'payer_status',
                 'contact_phone',
                 'residence_country'
                 ],
        }),
        ("Seller", {
            "description": "The information about the Seller.",
            'classes': ('collapse',),
            "fields":
                ['business',
                 'item_name',
                 'item_number',
                 'quantity',
                 'receiver_email',
                 'receiver_id',
                 'custom',
                 'invoice',
                 'memo',
                 ],
        }),
        ("Subscriber", {
            "description": "The information about the Subscription.",
            'classes': ('collapse',),
            "fields":
                ['subscr_id',
                 'subscr_date',
                 'subscr_effective',
                 ],
        }),
        ("Recurring", {
            "description": "Information about recurring Payments.",
            "classes": ("collapse",),
            "fields":
                ['profile_status',
                 'initial_payment_amount',
                 'amount_per_cycle',
                 'outstanding_balance',
                 'period_type',
                 'product_name',
                 'product_type',
                 'recurring_payment_id',
                 'receipt_id',
                 'next_payment_date',
                 ],
        }),
        ("Admin", {
            "description": "Additional Info.",
            "classes": ('collapse',),
            "fields":
                ['test_ipn',
                 'ipaddress',
                 'query',
                 'flag_code',
                 'flag_info',
                 ],
        }),
    )
    list_display = ["__unicode__",
                    "flag",
                    "invoice",
                    "custom",
                    "payment_status",
                    "created_at",
                    ]
    search_fields = ["txn_id",
                     "recurring_payment_id",
                     ]


admin.site.register(PayPalPDT, PayPalPDTAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import requests

from paypal.standard.ipn.signals import invalid_ipn_received, valid_ipn_received
from paypal.standard.models import PayPalStandardBase


class PayPalIPN(PayPalStandardBase):
    """Logs PayPal IPN interactions."""
    format = u"<IPN: %s %s>"

    class Meta:
        db_table = "paypal_ipn"
        verbose_name = "PayPal IPN"

    def _postback(self):
        """Perform PayPal Postback validation."""
        return requests.post(self.get_endpoint(), data=b"cmd=_notify-validate&" + self.query.encode("ascii")).content

    def _verify_postback(self):
        if self.response != "VERIFIED":
            self.set_flag("Invalid postback. ({0})".format(self.response))

    def send_signals(self):
        """Shout for the world to hear whether a txn was successful."""
        if self.flag:
            invalid_ipn_received.send(sender=self)
            return
        else:
            valid_ipn_received.send(sender=self)

    def __repr__(self):
        return '<PayPalIPN id:{0}>'.format(self.id)

    def __str__(self):
        return "PayPalIPN: {0}".format(self.id)



# Generated by Django 2.1.3 on 2018-11-28 10:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pro', '0002_auto_20141117_1647'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paypalnvp',
            name='flag',
            field=models.BooleanField(blank=True, default=False),
        ),
    ]



#!/usr/bin/env python

import os.path

from setuptools import find_packages, setup


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


DESCRIPTION = 'A pluggable Django application for integrating PayPal Payments Standard or Payments Pro'
URL = 'https://github.com/spookylukey/django-paypal'
DOCS_URL = 'https://django-paypal.readthedocs.org'

setup(
    name='django-paypal',
    version="1.1.1",
    author='John Boxall',
    author_email='john@handimobility.ca',
    maintainer="Luke Plant",
    maintainer_email="L.Plant.98@cantab.net",
    url=URL,
    install_requires=[
        'Django>=1.11',
        'six>=1.4.1',
        'requests>=2.5.3',
        'pytz>=2015.4',
    ],
    description=DESCRIPTION,
    long_description="%s\n\nHome page: %s\n\nDocs: %s\n" % (DESCRIPTION, URL, DOCS_URL),
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Framework :: Django",
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
